# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class SamlEntityDescriptor(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'additional_metadata_locations': 'list[str]',
        'affiliation_descriptor': 'SamlAffiliationDescriptor',
        'attribute_authority_descriptors': 'list[SamlAttributeAuthorityDescriptor]',
        'authn_authority_descriptors': 'list[SamlAuthnAuthorityDescriptor]',
        'cache_duration': 'str',
        'contact_person': 'SamlContactPerson',
        'entity_id': 'str',
        'id': 'str',
        'idpsso_descriptors': 'list[SamlIDPSSODescriptor]',
        'organization': 'SamlOrganization',
        'pdp_descriptors': 'list[SamlPDPDescriptor]',
        'role_descriptors': 'list[SamlRoleDescriptor]',
        'spsso_descriptors': 'list[SamlSPSSODescriptor]',
        'signature': 'EtreeElement',
        'valid_until': 'datetime',
        'xml_name': 'XmlName'
    }

    attribute_map = {
        'additional_metadata_locations': 'AdditionalMetadataLocations',
        'affiliation_descriptor': 'AffiliationDescriptor',
        'attribute_authority_descriptors': 'AttributeAuthorityDescriptors',
        'authn_authority_descriptors': 'AuthnAuthorityDescriptors',
        'cache_duration': 'CacheDuration',
        'contact_person': 'ContactPerson',
        'entity_id': 'EntityID',
        'id': 'ID',
        'idpsso_descriptors': 'IDPSSODescriptors',
        'organization': 'Organization',
        'pdp_descriptors': 'PDPDescriptors',
        'role_descriptors': 'RoleDescriptors',
        'spsso_descriptors': 'SPSSODescriptors',
        'signature': 'Signature',
        'valid_until': 'ValidUntil',
        'xml_name': 'XMLName'
    }

    def __init__(self, additional_metadata_locations=None, affiliation_descriptor=None, attribute_authority_descriptors=None, authn_authority_descriptors=None, cache_duration=None, contact_person=None, entity_id=None, id=None, idpsso_descriptors=None, organization=None, pdp_descriptors=None, role_descriptors=None, spsso_descriptors=None, signature=None, valid_until=None, xml_name=None, local_vars_configuration=None):  # noqa: E501
        """SamlEntityDescriptor - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._additional_metadata_locations = None
        self._affiliation_descriptor = None
        self._attribute_authority_descriptors = None
        self._authn_authority_descriptors = None
        self._cache_duration = None
        self._contact_person = None
        self._entity_id = None
        self._id = None
        self._idpsso_descriptors = None
        self._organization = None
        self._pdp_descriptors = None
        self._role_descriptors = None
        self._spsso_descriptors = None
        self._signature = None
        self._valid_until = None
        self._xml_name = None
        self.discriminator = None

        if additional_metadata_locations is not None:
            self.additional_metadata_locations = additional_metadata_locations
        if affiliation_descriptor is not None:
            self.affiliation_descriptor = affiliation_descriptor
        if attribute_authority_descriptors is not None:
            self.attribute_authority_descriptors = attribute_authority_descriptors
        if authn_authority_descriptors is not None:
            self.authn_authority_descriptors = authn_authority_descriptors
        if cache_duration is not None:
            self.cache_duration = cache_duration
        if contact_person is not None:
            self.contact_person = contact_person
        if entity_id is not None:
            self.entity_id = entity_id
        if id is not None:
            self.id = id
        if idpsso_descriptors is not None:
            self.idpsso_descriptors = idpsso_descriptors
        if organization is not None:
            self.organization = organization
        if pdp_descriptors is not None:
            self.pdp_descriptors = pdp_descriptors
        if role_descriptors is not None:
            self.role_descriptors = role_descriptors
        if spsso_descriptors is not None:
            self.spsso_descriptors = spsso_descriptors
        if signature is not None:
            self.signature = signature
        if valid_until is not None:
            self.valid_until = valid_until
        if xml_name is not None:
            self.xml_name = xml_name

    @property
    def additional_metadata_locations(self):
        """Gets the additional_metadata_locations of this SamlEntityDescriptor.  # noqa: E501


        :return: The additional_metadata_locations of this SamlEntityDescriptor.  # noqa: E501
        :rtype: list[str]
        """
        return self._additional_metadata_locations

    @additional_metadata_locations.setter
    def additional_metadata_locations(self, additional_metadata_locations):
        """Sets the additional_metadata_locations of this SamlEntityDescriptor.


        :param additional_metadata_locations: The additional_metadata_locations of this SamlEntityDescriptor.  # noqa: E501
        :type additional_metadata_locations: list[str]
        """

        self._additional_metadata_locations = additional_metadata_locations

    @property
    def affiliation_descriptor(self):
        """Gets the affiliation_descriptor of this SamlEntityDescriptor.  # noqa: E501


        :return: The affiliation_descriptor of this SamlEntityDescriptor.  # noqa: E501
        :rtype: SamlAffiliationDescriptor
        """
        return self._affiliation_descriptor

    @affiliation_descriptor.setter
    def affiliation_descriptor(self, affiliation_descriptor):
        """Sets the affiliation_descriptor of this SamlEntityDescriptor.


        :param affiliation_descriptor: The affiliation_descriptor of this SamlEntityDescriptor.  # noqa: E501
        :type affiliation_descriptor: SamlAffiliationDescriptor
        """

        self._affiliation_descriptor = affiliation_descriptor

    @property
    def attribute_authority_descriptors(self):
        """Gets the attribute_authority_descriptors of this SamlEntityDescriptor.  # noqa: E501


        :return: The attribute_authority_descriptors of this SamlEntityDescriptor.  # noqa: E501
        :rtype: list[SamlAttributeAuthorityDescriptor]
        """
        return self._attribute_authority_descriptors

    @attribute_authority_descriptors.setter
    def attribute_authority_descriptors(self, attribute_authority_descriptors):
        """Sets the attribute_authority_descriptors of this SamlEntityDescriptor.


        :param attribute_authority_descriptors: The attribute_authority_descriptors of this SamlEntityDescriptor.  # noqa: E501
        :type attribute_authority_descriptors: list[SamlAttributeAuthorityDescriptor]
        """

        self._attribute_authority_descriptors = attribute_authority_descriptors

    @property
    def authn_authority_descriptors(self):
        """Gets the authn_authority_descriptors of this SamlEntityDescriptor.  # noqa: E501


        :return: The authn_authority_descriptors of this SamlEntityDescriptor.  # noqa: E501
        :rtype: list[SamlAuthnAuthorityDescriptor]
        """
        return self._authn_authority_descriptors

    @authn_authority_descriptors.setter
    def authn_authority_descriptors(self, authn_authority_descriptors):
        """Sets the authn_authority_descriptors of this SamlEntityDescriptor.


        :param authn_authority_descriptors: The authn_authority_descriptors of this SamlEntityDescriptor.  # noqa: E501
        :type authn_authority_descriptors: list[SamlAuthnAuthorityDescriptor]
        """

        self._authn_authority_descriptors = authn_authority_descriptors

    @property
    def cache_duration(self):
        """Gets the cache_duration of this SamlEntityDescriptor.  # noqa: E501


        :return: The cache_duration of this SamlEntityDescriptor.  # noqa: E501
        :rtype: str
        """
        return self._cache_duration

    @cache_duration.setter
    def cache_duration(self, cache_duration):
        """Sets the cache_duration of this SamlEntityDescriptor.


        :param cache_duration: The cache_duration of this SamlEntityDescriptor.  # noqa: E501
        :type cache_duration: str
        """

        self._cache_duration = cache_duration

    @property
    def contact_person(self):
        """Gets the contact_person of this SamlEntityDescriptor.  # noqa: E501


        :return: The contact_person of this SamlEntityDescriptor.  # noqa: E501
        :rtype: SamlContactPerson
        """
        return self._contact_person

    @contact_person.setter
    def contact_person(self, contact_person):
        """Sets the contact_person of this SamlEntityDescriptor.


        :param contact_person: The contact_person of this SamlEntityDescriptor.  # noqa: E501
        :type contact_person: SamlContactPerson
        """

        self._contact_person = contact_person

    @property
    def entity_id(self):
        """Gets the entity_id of this SamlEntityDescriptor.  # noqa: E501


        :return: The entity_id of this SamlEntityDescriptor.  # noqa: E501
        :rtype: str
        """
        return self._entity_id

    @entity_id.setter
    def entity_id(self, entity_id):
        """Sets the entity_id of this SamlEntityDescriptor.


        :param entity_id: The entity_id of this SamlEntityDescriptor.  # noqa: E501
        :type entity_id: str
        """

        self._entity_id = entity_id

    @property
    def id(self):
        """Gets the id of this SamlEntityDescriptor.  # noqa: E501


        :return: The id of this SamlEntityDescriptor.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SamlEntityDescriptor.


        :param id: The id of this SamlEntityDescriptor.  # noqa: E501
        :type id: str
        """

        self._id = id

    @property
    def idpsso_descriptors(self):
        """Gets the idpsso_descriptors of this SamlEntityDescriptor.  # noqa: E501


        :return: The idpsso_descriptors of this SamlEntityDescriptor.  # noqa: E501
        :rtype: list[SamlIDPSSODescriptor]
        """
        return self._idpsso_descriptors

    @idpsso_descriptors.setter
    def idpsso_descriptors(self, idpsso_descriptors):
        """Sets the idpsso_descriptors of this SamlEntityDescriptor.


        :param idpsso_descriptors: The idpsso_descriptors of this SamlEntityDescriptor.  # noqa: E501
        :type idpsso_descriptors: list[SamlIDPSSODescriptor]
        """

        self._idpsso_descriptors = idpsso_descriptors

    @property
    def organization(self):
        """Gets the organization of this SamlEntityDescriptor.  # noqa: E501


        :return: The organization of this SamlEntityDescriptor.  # noqa: E501
        :rtype: SamlOrganization
        """
        return self._organization

    @organization.setter
    def organization(self, organization):
        """Sets the organization of this SamlEntityDescriptor.


        :param organization: The organization of this SamlEntityDescriptor.  # noqa: E501
        :type organization: SamlOrganization
        """

        self._organization = organization

    @property
    def pdp_descriptors(self):
        """Gets the pdp_descriptors of this SamlEntityDescriptor.  # noqa: E501


        :return: The pdp_descriptors of this SamlEntityDescriptor.  # noqa: E501
        :rtype: list[SamlPDPDescriptor]
        """
        return self._pdp_descriptors

    @pdp_descriptors.setter
    def pdp_descriptors(self, pdp_descriptors):
        """Sets the pdp_descriptors of this SamlEntityDescriptor.


        :param pdp_descriptors: The pdp_descriptors of this SamlEntityDescriptor.  # noqa: E501
        :type pdp_descriptors: list[SamlPDPDescriptor]
        """

        self._pdp_descriptors = pdp_descriptors

    @property
    def role_descriptors(self):
        """Gets the role_descriptors of this SamlEntityDescriptor.  # noqa: E501


        :return: The role_descriptors of this SamlEntityDescriptor.  # noqa: E501
        :rtype: list[SamlRoleDescriptor]
        """
        return self._role_descriptors

    @role_descriptors.setter
    def role_descriptors(self, role_descriptors):
        """Sets the role_descriptors of this SamlEntityDescriptor.


        :param role_descriptors: The role_descriptors of this SamlEntityDescriptor.  # noqa: E501
        :type role_descriptors: list[SamlRoleDescriptor]
        """

        self._role_descriptors = role_descriptors

    @property
    def spsso_descriptors(self):
        """Gets the spsso_descriptors of this SamlEntityDescriptor.  # noqa: E501


        :return: The spsso_descriptors of this SamlEntityDescriptor.  # noqa: E501
        :rtype: list[SamlSPSSODescriptor]
        """
        return self._spsso_descriptors

    @spsso_descriptors.setter
    def spsso_descriptors(self, spsso_descriptors):
        """Sets the spsso_descriptors of this SamlEntityDescriptor.


        :param spsso_descriptors: The spsso_descriptors of this SamlEntityDescriptor.  # noqa: E501
        :type spsso_descriptors: list[SamlSPSSODescriptor]
        """

        self._spsso_descriptors = spsso_descriptors

    @property
    def signature(self):
        """Gets the signature of this SamlEntityDescriptor.  # noqa: E501


        :return: The signature of this SamlEntityDescriptor.  # noqa: E501
        :rtype: EtreeElement
        """
        return self._signature

    @signature.setter
    def signature(self, signature):
        """Sets the signature of this SamlEntityDescriptor.


        :param signature: The signature of this SamlEntityDescriptor.  # noqa: E501
        :type signature: EtreeElement
        """

        self._signature = signature

    @property
    def valid_until(self):
        """Gets the valid_until of this SamlEntityDescriptor.  # noqa: E501


        :return: The valid_until of this SamlEntityDescriptor.  # noqa: E501
        :rtype: datetime
        """
        return self._valid_until

    @valid_until.setter
    def valid_until(self, valid_until):
        """Sets the valid_until of this SamlEntityDescriptor.


        :param valid_until: The valid_until of this SamlEntityDescriptor.  # noqa: E501
        :type valid_until: datetime
        """

        self._valid_until = valid_until

    @property
    def xml_name(self):
        """Gets the xml_name of this SamlEntityDescriptor.  # noqa: E501


        :return: The xml_name of this SamlEntityDescriptor.  # noqa: E501
        :rtype: XmlName
        """
        return self._xml_name

    @xml_name.setter
    def xml_name(self, xml_name):
        """Sets the xml_name of this SamlEntityDescriptor.


        :param xml_name: The xml_name of this SamlEntityDescriptor.  # noqa: E501
        :type xml_name: XmlName
        """

        self._xml_name = xml_name

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SamlEntityDescriptor):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SamlEntityDescriptor):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class OrmKernelClusterEdges(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'cluster_quotas': 'list[OrmClusterQuota]',
        'default_storage': 'OrmStorage',
        'kernel_cluster_nodes': 'list[OrmKernelClusterNode]',
        'kernel_cluster_storages': 'list[OrmKernelClusterStorage]',
        'kernel_cluster_workloads': 'list[OrmWorkload]',
        'kernel_resource_specs': 'list[OrmKernelResourceSpec]',
        'llm_knowledge': 'list[OrmLLMKnowledge]',
        'model_services': 'list[OrmModelService]',
        'organization_kernel_clusters': 'list[OrmOrganizationKernelCluster]',
        'primary_owner': 'OrmOrganization',
        'region': 'OrmRegion',
        'run_specs': 'list[OrmRunSpec]',
        'withdraw_history_kernel_clusters': 'list[OrmWithdrawHistoryKernelCluster]'
    }

    attribute_map = {
        'cluster_quotas': 'cluster_quotas',
        'default_storage': 'default_storage',
        'kernel_cluster_nodes': 'kernel_cluster_nodes',
        'kernel_cluster_storages': 'kernel_cluster_storages',
        'kernel_cluster_workloads': 'kernel_cluster_workloads',
        'kernel_resource_specs': 'kernel_resource_specs',
        'llm_knowledge': 'llm_knowledge',
        'model_services': 'model_services',
        'organization_kernel_clusters': 'organization_kernel_clusters',
        'primary_owner': 'primary_owner',
        'region': 'region',
        'run_specs': 'run_specs',
        'withdraw_history_kernel_clusters': 'withdraw_history_kernel_clusters'
    }

    def __init__(self, cluster_quotas=None, default_storage=None, kernel_cluster_nodes=None, kernel_cluster_storages=None, kernel_cluster_workloads=None, kernel_resource_specs=None, llm_knowledge=None, model_services=None, organization_kernel_clusters=None, primary_owner=None, region=None, run_specs=None, withdraw_history_kernel_clusters=None, local_vars_configuration=None):  # noqa: E501
        """OrmKernelClusterEdges - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._cluster_quotas = None
        self._default_storage = None
        self._kernel_cluster_nodes = None
        self._kernel_cluster_storages = None
        self._kernel_cluster_workloads = None
        self._kernel_resource_specs = None
        self._llm_knowledge = None
        self._model_services = None
        self._organization_kernel_clusters = None
        self._primary_owner = None
        self._region = None
        self._run_specs = None
        self._withdraw_history_kernel_clusters = None
        self.discriminator = None

        if cluster_quotas is not None:
            self.cluster_quotas = cluster_quotas
        if default_storage is not None:
            self.default_storage = default_storage
        if kernel_cluster_nodes is not None:
            self.kernel_cluster_nodes = kernel_cluster_nodes
        if kernel_cluster_storages is not None:
            self.kernel_cluster_storages = kernel_cluster_storages
        if kernel_cluster_workloads is not None:
            self.kernel_cluster_workloads = kernel_cluster_workloads
        if kernel_resource_specs is not None:
            self.kernel_resource_specs = kernel_resource_specs
        if llm_knowledge is not None:
            self.llm_knowledge = llm_knowledge
        if model_services is not None:
            self.model_services = model_services
        if organization_kernel_clusters is not None:
            self.organization_kernel_clusters = organization_kernel_clusters
        if primary_owner is not None:
            self.primary_owner = primary_owner
        if region is not None:
            self.region = region
        if run_specs is not None:
            self.run_specs = run_specs
        if withdraw_history_kernel_clusters is not None:
            self.withdraw_history_kernel_clusters = withdraw_history_kernel_clusters

    @property
    def cluster_quotas(self):
        """Gets the cluster_quotas of this OrmKernelClusterEdges.  # noqa: E501


        :return: The cluster_quotas of this OrmKernelClusterEdges.  # noqa: E501
        :rtype: list[OrmClusterQuota]
        """
        return self._cluster_quotas

    @cluster_quotas.setter
    def cluster_quotas(self, cluster_quotas):
        """Sets the cluster_quotas of this OrmKernelClusterEdges.


        :param cluster_quotas: The cluster_quotas of this OrmKernelClusterEdges.  # noqa: E501
        :type cluster_quotas: list[OrmClusterQuota]
        """

        self._cluster_quotas = cluster_quotas

    @property
    def default_storage(self):
        """Gets the default_storage of this OrmKernelClusterEdges.  # noqa: E501


        :return: The default_storage of this OrmKernelClusterEdges.  # noqa: E501
        :rtype: OrmStorage
        """
        return self._default_storage

    @default_storage.setter
    def default_storage(self, default_storage):
        """Sets the default_storage of this OrmKernelClusterEdges.


        :param default_storage: The default_storage of this OrmKernelClusterEdges.  # noqa: E501
        :type default_storage: OrmStorage
        """

        self._default_storage = default_storage

    @property
    def kernel_cluster_nodes(self):
        """Gets the kernel_cluster_nodes of this OrmKernelClusterEdges.  # noqa: E501


        :return: The kernel_cluster_nodes of this OrmKernelClusterEdges.  # noqa: E501
        :rtype: list[OrmKernelClusterNode]
        """
        return self._kernel_cluster_nodes

    @kernel_cluster_nodes.setter
    def kernel_cluster_nodes(self, kernel_cluster_nodes):
        """Sets the kernel_cluster_nodes of this OrmKernelClusterEdges.


        :param kernel_cluster_nodes: The kernel_cluster_nodes of this OrmKernelClusterEdges.  # noqa: E501
        :type kernel_cluster_nodes: list[OrmKernelClusterNode]
        """

        self._kernel_cluster_nodes = kernel_cluster_nodes

    @property
    def kernel_cluster_storages(self):
        """Gets the kernel_cluster_storages of this OrmKernelClusterEdges.  # noqa: E501


        :return: The kernel_cluster_storages of this OrmKernelClusterEdges.  # noqa: E501
        :rtype: list[OrmKernelClusterStorage]
        """
        return self._kernel_cluster_storages

    @kernel_cluster_storages.setter
    def kernel_cluster_storages(self, kernel_cluster_storages):
        """Sets the kernel_cluster_storages of this OrmKernelClusterEdges.


        :param kernel_cluster_storages: The kernel_cluster_storages of this OrmKernelClusterEdges.  # noqa: E501
        :type kernel_cluster_storages: list[OrmKernelClusterStorage]
        """

        self._kernel_cluster_storages = kernel_cluster_storages

    @property
    def kernel_cluster_workloads(self):
        """Gets the kernel_cluster_workloads of this OrmKernelClusterEdges.  # noqa: E501


        :return: The kernel_cluster_workloads of this OrmKernelClusterEdges.  # noqa: E501
        :rtype: list[OrmWorkload]
        """
        return self._kernel_cluster_workloads

    @kernel_cluster_workloads.setter
    def kernel_cluster_workloads(self, kernel_cluster_workloads):
        """Sets the kernel_cluster_workloads of this OrmKernelClusterEdges.


        :param kernel_cluster_workloads: The kernel_cluster_workloads of this OrmKernelClusterEdges.  # noqa: E501
        :type kernel_cluster_workloads: list[OrmWorkload]
        """

        self._kernel_cluster_workloads = kernel_cluster_workloads

    @property
    def kernel_resource_specs(self):
        """Gets the kernel_resource_specs of this OrmKernelClusterEdges.  # noqa: E501


        :return: The kernel_resource_specs of this OrmKernelClusterEdges.  # noqa: E501
        :rtype: list[OrmKernelResourceSpec]
        """
        return self._kernel_resource_specs

    @kernel_resource_specs.setter
    def kernel_resource_specs(self, kernel_resource_specs):
        """Sets the kernel_resource_specs of this OrmKernelClusterEdges.


        :param kernel_resource_specs: The kernel_resource_specs of this OrmKernelClusterEdges.  # noqa: E501
        :type kernel_resource_specs: list[OrmKernelResourceSpec]
        """

        self._kernel_resource_specs = kernel_resource_specs

    @property
    def llm_knowledge(self):
        """Gets the llm_knowledge of this OrmKernelClusterEdges.  # noqa: E501


        :return: The llm_knowledge of this OrmKernelClusterEdges.  # noqa: E501
        :rtype: list[OrmLLMKnowledge]
        """
        return self._llm_knowledge

    @llm_knowledge.setter
    def llm_knowledge(self, llm_knowledge):
        """Sets the llm_knowledge of this OrmKernelClusterEdges.


        :param llm_knowledge: The llm_knowledge of this OrmKernelClusterEdges.  # noqa: E501
        :type llm_knowledge: list[OrmLLMKnowledge]
        """

        self._llm_knowledge = llm_knowledge

    @property
    def model_services(self):
        """Gets the model_services of this OrmKernelClusterEdges.  # noqa: E501


        :return: The model_services of this OrmKernelClusterEdges.  # noqa: E501
        :rtype: list[OrmModelService]
        """
        return self._model_services

    @model_services.setter
    def model_services(self, model_services):
        """Sets the model_services of this OrmKernelClusterEdges.


        :param model_services: The model_services of this OrmKernelClusterEdges.  # noqa: E501
        :type model_services: list[OrmModelService]
        """

        self._model_services = model_services

    @property
    def organization_kernel_clusters(self):
        """Gets the organization_kernel_clusters of this OrmKernelClusterEdges.  # noqa: E501


        :return: The organization_kernel_clusters of this OrmKernelClusterEdges.  # noqa: E501
        :rtype: list[OrmOrganizationKernelCluster]
        """
        return self._organization_kernel_clusters

    @organization_kernel_clusters.setter
    def organization_kernel_clusters(self, organization_kernel_clusters):
        """Sets the organization_kernel_clusters of this OrmKernelClusterEdges.


        :param organization_kernel_clusters: The organization_kernel_clusters of this OrmKernelClusterEdges.  # noqa: E501
        :type organization_kernel_clusters: list[OrmOrganizationKernelCluster]
        """

        self._organization_kernel_clusters = organization_kernel_clusters

    @property
    def primary_owner(self):
        """Gets the primary_owner of this OrmKernelClusterEdges.  # noqa: E501


        :return: The primary_owner of this OrmKernelClusterEdges.  # noqa: E501
        :rtype: OrmOrganization
        """
        return self._primary_owner

    @primary_owner.setter
    def primary_owner(self, primary_owner):
        """Sets the primary_owner of this OrmKernelClusterEdges.


        :param primary_owner: The primary_owner of this OrmKernelClusterEdges.  # noqa: E501
        :type primary_owner: OrmOrganization
        """

        self._primary_owner = primary_owner

    @property
    def region(self):
        """Gets the region of this OrmKernelClusterEdges.  # noqa: E501


        :return: The region of this OrmKernelClusterEdges.  # noqa: E501
        :rtype: OrmRegion
        """
        return self._region

    @region.setter
    def region(self, region):
        """Sets the region of this OrmKernelClusterEdges.


        :param region: The region of this OrmKernelClusterEdges.  # noqa: E501
        :type region: OrmRegion
        """

        self._region = region

    @property
    def run_specs(self):
        """Gets the run_specs of this OrmKernelClusterEdges.  # noqa: E501


        :return: The run_specs of this OrmKernelClusterEdges.  # noqa: E501
        :rtype: list[OrmRunSpec]
        """
        return self._run_specs

    @run_specs.setter
    def run_specs(self, run_specs):
        """Sets the run_specs of this OrmKernelClusterEdges.


        :param run_specs: The run_specs of this OrmKernelClusterEdges.  # noqa: E501
        :type run_specs: list[OrmRunSpec]
        """

        self._run_specs = run_specs

    @property
    def withdraw_history_kernel_clusters(self):
        """Gets the withdraw_history_kernel_clusters of this OrmKernelClusterEdges.  # noqa: E501


        :return: The withdraw_history_kernel_clusters of this OrmKernelClusterEdges.  # noqa: E501
        :rtype: list[OrmWithdrawHistoryKernelCluster]
        """
        return self._withdraw_history_kernel_clusters

    @withdraw_history_kernel_clusters.setter
    def withdraw_history_kernel_clusters(self, withdraw_history_kernel_clusters):
        """Sets the withdraw_history_kernel_clusters of this OrmKernelClusterEdges.


        :param withdraw_history_kernel_clusters: The withdraw_history_kernel_clusters of this OrmKernelClusterEdges.  # noqa: E501
        :type withdraw_history_kernel_clusters: list[OrmWithdrawHistoryKernelCluster]
        """

        self._withdraw_history_kernel_clusters = withdraw_history_kernel_clusters

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmKernelClusterEdges):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmKernelClusterEdges):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class OrmDatasetEdges(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'organization': 'OrmOrganization',
        'project_datasets': 'list[OrmProjectDataset]',
        'summaries': 'list[OrmDatasetSummary]',
        'version_volume': 'OrmVolume',
        'volume': 'OrmVolume'
    }

    attribute_map = {
        'organization': 'organization',
        'project_datasets': 'project_datasets',
        'summaries': 'summaries',
        'version_volume': 'version_volume',
        'volume': 'volume'
    }

    def __init__(self, organization=None, project_datasets=None, summaries=None, version_volume=None, volume=None, local_vars_configuration=None):  # noqa: E501
        """OrmDatasetEdges - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._organization = None
        self._project_datasets = None
        self._summaries = None
        self._version_volume = None
        self._volume = None
        self.discriminator = None

        if organization is not None:
            self.organization = organization
        if project_datasets is not None:
            self.project_datasets = project_datasets
        if summaries is not None:
            self.summaries = summaries
        if version_volume is not None:
            self.version_volume = version_volume
        if volume is not None:
            self.volume = volume

    @property
    def organization(self):
        """Gets the organization of this OrmDatasetEdges.  # noqa: E501


        :return: The organization of this OrmDatasetEdges.  # noqa: E501
        :rtype: OrmOrganization
        """
        return self._organization

    @organization.setter
    def organization(self, organization):
        """Sets the organization of this OrmDatasetEdges.


        :param organization: The organization of this OrmDatasetEdges.  # noqa: E501
        :type organization: OrmOrganization
        """

        self._organization = organization

    @property
    def project_datasets(self):
        """Gets the project_datasets of this OrmDatasetEdges.  # noqa: E501


        :return: The project_datasets of this OrmDatasetEdges.  # noqa: E501
        :rtype: list[OrmProjectDataset]
        """
        return self._project_datasets

    @project_datasets.setter
    def project_datasets(self, project_datasets):
        """Sets the project_datasets of this OrmDatasetEdges.


        :param project_datasets: The project_datasets of this OrmDatasetEdges.  # noqa: E501
        :type project_datasets: list[OrmProjectDataset]
        """

        self._project_datasets = project_datasets

    @property
    def summaries(self):
        """Gets the summaries of this OrmDatasetEdges.  # noqa: E501


        :return: The summaries of this OrmDatasetEdges.  # noqa: E501
        :rtype: list[OrmDatasetSummary]
        """
        return self._summaries

    @summaries.setter
    def summaries(self, summaries):
        """Sets the summaries of this OrmDatasetEdges.


        :param summaries: The summaries of this OrmDatasetEdges.  # noqa: E501
        :type summaries: list[OrmDatasetSummary]
        """

        self._summaries = summaries

    @property
    def version_volume(self):
        """Gets the version_volume of this OrmDatasetEdges.  # noqa: E501


        :return: The version_volume of this OrmDatasetEdges.  # noqa: E501
        :rtype: OrmVolume
        """
        return self._version_volume

    @version_volume.setter
    def version_volume(self, version_volume):
        """Sets the version_volume of this OrmDatasetEdges.


        :param version_volume: The version_volume of this OrmDatasetEdges.  # noqa: E501
        :type version_volume: OrmVolume
        """

        self._version_volume = version_volume

    @property
    def volume(self):
        """Gets the volume of this OrmDatasetEdges.  # noqa: E501


        :return: The volume of this OrmDatasetEdges.  # noqa: E501
        :rtype: OrmVolume
        """
        return self._volume

    @volume.setter
    def volume(self, volume):
        """Sets the volume of this OrmDatasetEdges.


        :param volume: The volume of this OrmDatasetEdges.  # noqa: E501
        :type volume: OrmVolume
        """

        self._volume = volume

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmDatasetEdges):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmDatasetEdges):
            return True

        return self.to_dict() != other.to_dict()



from typing import Optional, Literal

from vessl.openapi_client import ProtoLLMVectorDBConnAuth


class ConnectionAuthUserPass(object):
    username: str
    password: str
    def __init__(self, username, password):
        self.username = username
        self.password = password

class ConnectionAuthBearer(object):
    token: str
    def __init__(self, token):
        self.token = token

class ConnectionAuthHeader(object):
    header_key: str
    header_value: str
    def __init__(self, header_key, header_value):
        self.header_key = header_key
        self.header_value = header_value

class ConnectionAuth(object):
    auth_type: Literal["userpass", "bearer", "header", "none"]
    userpass: Optional[ConnectionAuthUserPass]
    bearer: Optional[ConnectionAuthBearer]
    header: Optional[ConnectionAuthHeader]

    def __init__(self, auth: ProtoLLMVectorDBConnAuth):
        if auth.type == "none":
            self.auth_type = "none"
        elif auth.type == "userpass":
            self.auth_type = "userpass"
            self.userpass = ConnectionAuthUserPass(auth.username, auth.password)
        elif auth.type == "bearer":
            self.auth_type = "bearer"
            self.bearer = ConnectionAuthBearer(auth.token)
        elif auth.type == "header":
            self.auth_type = "header"
            self.header = ConnectionAuthHeader(auth.header_key, auth.header_value)


from typing import Optional, Literal, Any

from vessl.openapi_client import V1WorkflowNodeStruct, V1WorkflowNodeInlineCode, V1BaseWorkflowNode, V1WorkflowNodeLLMQuery, \
    V1WorkflowNodeLLMQueryRequest, V1WorkflowNodeVectorSearch, V1WorkflowNodeVectorSearchRequest, V1SingleWorkflowEdge, \
    V1WorkflowNodeInput, V1SingleLLMQueryMessage, V1SingleWorkflowNodeOutput, V1WorkflowRevisionInput, \
    V1VariableValueInput, V1SingleWorkflowEdgeInput
from vessl.llm import connection


class WorkflowEdge(object):
    when: Optional[str]
    otherwise: Optional[bool]
    next_node: str

    def __init__(self, when: Optional[str], otherwise: Optional[bool], next_node: str):
        self.when = when
        self.otherwise = otherwise
        self.next_node = next_node

    @classmethod
    def from_config(cls, edge: V1SingleWorkflowEdge):
        return cls(edge.when, edge.otherwise, edge.next_node)

    @classmethod
    def from_dict(cls, payload: dict[str, Any]):
        return cls(payload.get("when"), payload.get("otherwise"), payload.get("next_node"))

    def to_v1_single_workflow_edge_input(self) -> V1SingleWorkflowEdgeInput:
        return V1SingleWorkflowEdgeInput(
            when=self.when,
            otherwise=self.otherwise,
            next_node=self.next_node
        )


class SingleNodeOutput(object):
    text: Optional[str]
    expr: Optional[str]

    def __init__(self, text: Optional[str], expr: Optional[str]):
        self.text = text
        self.expr = expr

    @classmethod
    def from_config(cls, output: V1SingleWorkflowNodeOutput):
        return cls(output.text, output.expr)

    @classmethod
    def from_dict(cls, payload: dict[str, Any]):
        return cls(payload.get("text"), payload.get("expr"))

    def to_v1_single_workflow_node_output(self) -> V1SingleWorkflowNodeOutput:
        return V1SingleWorkflowNodeOutput(
            text=self.text,
            expr=self.expr
        )


class BaseWorkflowNode(object):
    title: str
    outputs: dict[str, SingleNodeOutput]

    def __init__(self, title: str, outputs: dict[str, SingleNodeOutput]):
        self.title = title
        self.outputs = outputs

    def to_v1_base_workflow_node(self) -> V1BaseWorkflowNode:
        return V1BaseWorkflowNode(title=self.title, outputs={
            k: v.to_v1_single_workflow_node_output() for k, v in self.outputs.items()
        })


class WorkflowNodeInlineCode(BaseWorkflowNode):
    code: str

    def __init__(self, code: str, base_config: Optional[V1BaseWorkflowNode] = None,
                 base_dict: Optional[dict[str, Any]] = None):
        if base_config is not None:
            if base_config.outputs is not None:
                outputs = {k: SingleNodeOutput.from_config(v) for k, v in base_config.outputs.items()}
            else:
                outputs = dict()
            super().__init__(base_config.title, outputs)
        else:
            outputs = {k: SingleNodeOutput.from_dict(v) for k, v in base_dict.get("outputs", {}).items()}
            super().__init__(base_dict.get("title"), outputs)
        self.code = code

    @classmethod
    def from_config(cls, inline_code: V1WorkflowNodeInlineCode):
        return cls(code=inline_code.code, base_config=inline_code.base_workflow_node)

    @classmethod
    def from_dict(cls, payload: dict[str, Any]):
        return cls(code=payload.get("code"), base_dict=payload.get("base_workflow_node", {}))

    def to_v1_workflow_node_inline_code(self) -> V1WorkflowNodeInlineCode:
        return V1WorkflowNodeInlineCode(
            base_workflow_node=self.to_v1_base_workflow_node(),
            code=self.code
        )


class LLMQueryMessage(object):
    RoleSystem = "system"
    RoleUser = "user"
    RoleAgent = "agent"
    RoleType = Literal["system", "user", "agent"]
    role: RoleType
    content: str

    def __init__(self, role: RoleType, content: str):
        self.role = role
        self.content = content

    @classmethod
    def from_config(cls, message: V1SingleLLMQueryMessage):
        role: Literal["system", "user", "agent"] = "system"
        if message.role == "user":
            role = "user"
        elif message.role == "agent":
            role = "agent"
        return cls(role, message.content)

    @classmethod
    def from_dict(cls, payload: dict[str, Any]):
        return cls(payload.get("role"), payload.get("content"))


class LLMQueryRequest(object):
    messages: list[LLMQueryMessage]
    payload: dict[str, Any]

    def __init__(self, messages: list[LLMQueryMessage], payload: dict[str, Any]):
        self.messages = messages
        self.payload = payload

    @classmethod
    def from_config(cls, request: V1WorkflowNodeLLMQueryRequest):
        messages = [LLMQueryMessage.from_config(message) for message in request.messages]
        return cls(messages, request.payload)

    @classmethod
    def from_dict(cls, payload: dict[str, Any]):
        messages = [LLMQueryMessage.from_dict(message) for message in payload.get("messages", [])]
        return cls(messages, payload.get("payload"))


class WorkflowNodeLLMQuery(BaseWorkflowNode):
    request: LLMQueryRequest
    llm_model: connection.ModelConnection
    model_connection_name: str

    def __init__(self, request: LLMQueryRequest, model_connection_name: str,
                 llm_model: Optional[connection.ModelConnection] = None,
                 base_config: Optional[V1BaseWorkflowNode] = None, base_dict: Optional[dict[str, Any]] = None):
        if base_config is not None:
            if base_config.outputs is not None:
                outputs = {k: SingleNodeOutput.from_config(v) for k, v in base_config.outputs.items()}
            else:
                outputs = dict()
            super().__init__(base_config.title, outputs)
        else:
            outputs = {k: SingleNodeOutput.from_dict(v) for k, v in base_dict.get("outputs", {}).items()}
            super().__init__(base_dict.get("title"), outputs)

        self.request = request
        self.llm_model = llm_model
        self.model_connection_name = model_connection_name

    @classmethod
    def from_config(cls, llm_query: V1WorkflowNodeLLMQuery, llm_model: connection.ModelConnection):
        base_workflow_node: V1BaseWorkflowNode = llm_query.base_workflow_node
        request = LLMQueryRequest.from_config(llm_query.request)
        model_connection_name = llm_query.model_connection_name
        return cls(request, model_connection_name, llm_model=llm_model, base_config=base_workflow_node)

    @classmethod
    def from_dict(cls, payload: dict[str, Any]):
        # only called when building input from any, thus no llm_model is passed
        request = LLMQueryRequest.from_dict(payload.get("request", {}))
        model_connection_name = payload.get("model_connection_name")
        return cls(request, model_connection_name, base_dict=payload.get("base_workflow_node", {}))

    def to_v1_workflow_node_llm_query(self) -> V1WorkflowNodeLLMQuery:
        messages = [
            V1SingleLLMQueryMessage(role=message.role, content=message.content) for message in self.request.messages
        ]
        return V1WorkflowNodeLLMQuery(
            base_workflow_node=self.to_v1_base_workflow_node(),
            request=V1WorkflowNodeLLMQueryRequest(
                messages=messages,
                payload=self.request.payload,
            ),
            model_connection_name=self.model_connection_name
        )


class VectorSearchRequest(object):
    input: str
    payload: dict[str, Any]

    def __init__(self, input_str: str, payload: dict[str, Any]):
        self.input = input_str
        self.payload = payload

    @classmethod
    def from_config(cls, request: V1WorkflowNodeVectorSearchRequest):
        return cls(request.input, request.payload)

    @classmethod
    def from_dict(cls, payload: dict[str, Any]):
        return cls(payload.get("input"), payload.get("payload"))


class WorkflowNodeVectorSearch(BaseWorkflowNode):
    request: VectorSearchRequest
    knowledge_name: str
    embedding_model: Optional[connection.ModelConnection]
    vector_db: Optional[connection.VectorDBConnection]
    permission: Optional[connection.PermissionConfig]

    def __init__(self, request: VectorSearchRequest, knowledge_name: str,
                 embedding_model: Optional[connection.ModelConnection] = None,
                 vector_db: Optional[connection.VectorDBConnection] = None,
                 permission: Optional[connection.PermissionConfig] = None,
                 base_config: Optional[V1BaseWorkflowNode] = None, base_dict: Optional[dict[str, Any]] = None):
        if base_config is not None:
            if base_config.outputs is not None:
                outputs = {k: SingleNodeOutput.from_config(v) for k, v in base_config.outputs.items()}
            else:
                outputs = dict()
            super().__init__(base_config.title, outputs)
        else:
            outputs = {k: SingleNodeOutput.from_dict(v) for k, v in base_dict.get("outputs", {}).items()}
            super().__init__(base_dict.get("title"), outputs)

        self.request = request
        self.knowledge_name = knowledge_name
        self.embedding_model = embedding_model
        self.vector_db = vector_db
        self.permission = permission

    @classmethod
    def from_config(cls, vector_search: V1WorkflowNodeVectorSearch, embedding_model: connection.ModelConnection,
                    vector_db: connection.VectorDBConnection, permission: connection.PermissionConfig):
        knowledge_name = vector_search.knowledge_name
        base_workflow_node: V1BaseWorkflowNode = vector_search.base_workflow_node
        request = VectorSearchRequest.from_config(vector_search.request)
        return cls(request, knowledge_name, embedding_model=embedding_model, vector_db=vector_db,
                   permission=permission, base_config=base_workflow_node)

    @classmethod
    def from_dict(cls, payload: dict[str, Any]):
        # only called when building input from any, thus no embedding_model, vector_db, permission is passed
        request = VectorSearchRequest.from_dict(payload.get("request", {}))
        knowledge_name = payload.get("knowledge_name")
        return cls(request, knowledge_name, base_dict=payload.get("base_workflow_node", {}))

    def to_v1_workflow_node_vector_search(self) -> V1WorkflowNodeVectorSearch:
        return V1WorkflowNodeVectorSearch(
            base_workflow_node=self.to_v1_base_workflow_node(),
            request=V1WorkflowNodeVectorSearchRequest(input=self.request.input, payload=self.request.payload),
            knowledge_name=self.knowledge_name,
        )


class WorkflowNode(object):
    NodeKind = Literal[
        "workflowv1/InlineCode", "workflowv1/LLMQuery", "workflowv1/VectorSearch", "workflowv1/InvokeTool"]
    NodeInlineCode: NodeKind = "workflowv1/InlineCode"
    NodeLLMQuery: NodeKind = "workflowv1/LLMQuery"
    NodeVectorSearch: NodeKind = "workflowv1/VectorSearch"
    NodeInvokeTool: NodeKind = "workflowv1/InvokeTool"
    kind: NodeKind
    vector_search: Optional[WorkflowNodeVectorSearch]
    llm_query: Optional[WorkflowNodeLLMQuery]
    inline_code: Optional[WorkflowNodeInlineCode]

    def __init__(self, kind: NodeKind,
                 vector_search: Optional[WorkflowNodeVectorSearch] = None,
                 llm_query: Optional[WorkflowNodeLLMQuery] = None,
                 inline_code: Optional[WorkflowNodeInlineCode] = None):
        self.kind = kind
        self.vector_search = vector_search
        self.llm_query = llm_query
        self.inline_code = inline_code

    @classmethod
    def from_config(cls, node: V1WorkflowNodeStruct, model_connection: Optional[connection.ModelConnection],
                    vector_db: Optional[connection.VectorDBConnection],
                    permission: Optional[connection.PermissionConfig]):

        if node.kind == cls.NodeInlineCode:
            return cls(kind=cls.NodeInlineCode, inline_code=WorkflowNodeInlineCode.from_config(node.inline_code))
        elif node.kind == cls.NodeLLMQuery:
            return cls(kind=cls.NodeLLMQuery, llm_query=WorkflowNodeLLMQuery.from_config(
                node.llm_query, model_connection,
            ))
        elif node.kind == cls.NodeVectorSearch:
            return cls(kind=cls.NodeVectorSearch, vector_search=WorkflowNodeVectorSearch.from_config(
                node.vector_search, model_connection, vector_db, permission,
            ))

    @classmethod
    def from_dict(cls, payload: dict[str, Any]):
        kind = payload.get("kind")
        if kind == cls.NodeInlineCode:
            inline_code = WorkflowNodeInlineCode.from_dict(payload.get("inline_code", {}))
            return WorkflowNode(kind=cls.NodeInlineCode, inline_code=inline_code)
        elif kind == cls.NodeLLMQuery:
            llm_query = WorkflowNodeLLMQuery.from_dict(payload.get("llm_query", {}))
            return WorkflowNode(kind=cls.NodeLLMQuery, llm_query=llm_query)
        elif kind == cls.NodeVectorSearch:
            vector_search = WorkflowNodeVectorSearch.from_dict(payload.get("vector_search", {}))
            return WorkflowNode(kind=cls.NodeVectorSearch, vector_search=vector_search)

    def to_v1_workflow_node_input(self) -> V1WorkflowNodeInput:
        return V1WorkflowNodeInput(
            kind=self.kind,
            inline_code=self.inline_code.to_v1_workflow_node_inline_code() if self.inline_code else None,
            llm_query=self.llm_query.to_v1_workflow_node_llm_query() if self.llm_query else None,
            vector_search=self.vector_search.to_v1_workflow_node_vector_search() if self.vector_search else None
        )


class SingleWorkflowVariable(object):
    source: Literal["text", "secret"]
    text: Optional[str]
    secret: Optional[str]
    resolved: Optional[str]

    def __init__(self, source: Literal["text", "secret"], text: Optional[str], secret: Optional[str],
                 resolved: Optional[str] = None):
        self.source = source
        self.text = text
        self.secret = secret
        self.resolved = resolved

    def to_v1_variable_value_input(self):
        return V1VariableValueInput(
            source=self.source,
            text=self.text,
            secret=self.secret,
        )


class Workflow(object):
    variables: dict[str, SingleWorkflowVariable]
    nodes: dict[str, WorkflowNode]
    edges: dict[str, list[WorkflowEdge]]
    permission_config: connection.PermissionConfig

    def __init__(self, nodes: dict[str, WorkflowNode], edges: dict[str, list[WorkflowEdge]],
                 variables: dict[str, SingleWorkflowVariable], permission_config: connection.PermissionConfig):
        self.nodes = nodes
        self.edges = edges
        self.variables = variables
        self.permission_config = permission_config


class WorkflowSpecInput(object):
    kind: Literal["workflowv1/Workflow"]
    variables: dict[str, SingleWorkflowVariable]
    nodes: dict[str, WorkflowNode]
    edges: dict[str, list[WorkflowEdge]]

    def __init__(self, kind: Literal["workflowv1/Workflow"], variables: dict[str, SingleWorkflowVariable],
                 nodes: dict[str, WorkflowNode], edges: dict[str, list[WorkflowEdge]]):
        self.kind = kind
        self.variables = variables
        self.nodes = nodes
        self.edges = edges

    @classmethod
    def from_dict(cls, spec_input: dict[str, Any]):
        raw_variables = spec_input.get("variables", {})
        variables = dict()
        for k, v in raw_variables.items():
            variables[k] = SingleWorkflowVariable(
                v["source"], v.get("text"), v.get("secret"), v.get("resolved"))
        raw_nodes = spec_input.get("nodes", {})
        nodes = dict()
        for k, v in raw_nodes.items():
            nodes[k] = WorkflowNode.from_dict(v)
        raw_edges = spec_input.get("edges", {})
        edges = dict()
        for k, v in raw_edges.items():
            edges[k] = [WorkflowEdge.from_dict(e) for e in v]
        return cls(kind="workflowv1/Workflow", variables=variables, nodes=nodes, edges=edges)

    def to_v1_workflow_revision_input(self):
        return V1WorkflowRevisionInput(
            kind=self.kind,
            variables={k: v.to_v1_variable_value_input() for k, v in self.variables.items()},
            nodes={k: v.to_v1_workflow_node_input() for k, v in self.nodes.items()},
            edges={k: [e.to_v1_single_workflow_edge_input() for e in edges] for k, edges in self.edges.items()},
        )



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class OrmUserEdges(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'access_tokens': 'list[OrmAccessToken]',
        'cluster_quotas': 'list[OrmClusterQuota]',
        'coupon_redemptions': 'list[OrmCouponRedemption]',
        'created_volume_v2s': 'list[OrmVolumeV2]',
        'credit_earn_histories': 'list[OrmCreditEarnHistory]',
        'default_organization': 'OrmOrganization',
        'experiments': 'list[OrmExperiment]',
        'hub_model_task_histories': 'list[OrmHubModelTaskHistory]',
        'hub_model_tasks': 'list[OrmHubModelTask]',
        'judged_pipeline_step_executions': 'list[OrmPipelineStepExecution]',
        'llm_api_keys': 'list[OrmLLMApiKey]',
        'llm_documents': 'list[OrmLLMDocument]',
        'llm_knowledge': 'list[OrmLLMKnowledge]',
        'llm_knowledge_jobs': 'list[OrmLLMKnowledgeJob]',
        'llm_model_connections': 'list[OrmLLMModelConnection]',
        'llm_model_parameter_definitions': 'list[OrmLLMModelParameterDefinition]',
        'llm_model_parameter_presets': 'list[OrmLLMModelParameterPreset]',
        'llm_user_groups': 'list[OrmLLMUserGroup]',
        'llm_vectordb_connections': 'list[OrmLLMVectorDBConnection]',
        'llm_workflow_revisions': 'list[OrmLLMWorkflowRevision]',
        'llm_workflows': 'list[OrmLLMWorkflow]',
        'memberships': 'list[OrmUserGroupMembership]',
        'model_service_revisions': 'list[OrmModelServiceRevision]',
        'model_service_rollouts': 'list[OrmModelServiceRollout]',
        'model_services': 'list[OrmModelService]',
        'models': 'list[OrmModel]',
        'onboardings': 'list[OrmOnboarding]',
        'owned_organizations': 'list[OrmOrganization]',
        'persistent_artifacts': 'list[OrmArtifact]',
        'pipelines': 'list[OrmPipeline]',
        'project_dashboards': 'list[OrmProjectDashboard]',
        'reset_password_tokens': 'list[OrmResetPasswordToken]',
        'run_executions': 'list[OrmRunExecution]',
        'run_tracking_dashboards': 'list[OrmRunTrackingDashboard]',
        'starred_dashboards': 'list[OrmProjectDashboard]',
        'triggered_pipeline_executions': 'list[OrmPipelineExecution]',
        'user_history_as_transferer': 'list[OrmUserHistory]',
        'user_organization': 'list[OrmUserOrganization]',
        'user_starred_projects': 'list[OrmUserStarredProjects]',
        'withdraw_history_organizations': 'list[OrmWithdrawHistoryOrganization]',
        'withdraw_history_workloads': 'list[OrmWithdrawHistoryWorkload]',
        'workspaces': 'list[OrmWorkspace]'
    }

    attribute_map = {
        'access_tokens': 'access_tokens',
        'cluster_quotas': 'cluster_quotas',
        'coupon_redemptions': 'coupon_redemptions',
        'created_volume_v2s': 'createdVolumeV2s',
        'credit_earn_histories': 'credit_earn_histories',
        'default_organization': 'default_organization',
        'experiments': 'experiments',
        'hub_model_task_histories': 'hub_model_task_histories',
        'hub_model_tasks': 'hub_model_tasks',
        'judged_pipeline_step_executions': 'judged_pipeline_step_executions',
        'llm_api_keys': 'llm_api_keys',
        'llm_documents': 'llm_documents',
        'llm_knowledge': 'llm_knowledge',
        'llm_knowledge_jobs': 'llm_knowledge_jobs',
        'llm_model_connections': 'llm_model_connections',
        'llm_model_parameter_definitions': 'llm_model_parameter_definitions',
        'llm_model_parameter_presets': 'llm_model_parameter_presets',
        'llm_user_groups': 'llm_user_groups',
        'llm_vectordb_connections': 'llm_vectordb_connections',
        'llm_workflow_revisions': 'llm_workflow_revisions',
        'llm_workflows': 'llm_workflows',
        'memberships': 'memberships',
        'model_service_revisions': 'model_service_revisions',
        'model_service_rollouts': 'model_service_rollouts',
        'model_services': 'model_services',
        'models': 'models',
        'onboardings': 'onboardings',
        'owned_organizations': 'owned_organizations',
        'persistent_artifacts': 'persistent_artifacts',
        'pipelines': 'pipelines',
        'project_dashboards': 'project_dashboards',
        'reset_password_tokens': 'reset_password_tokens',
        'run_executions': 'run_executions',
        'run_tracking_dashboards': 'run_tracking_dashboards',
        'starred_dashboards': 'starred_dashboards',
        'triggered_pipeline_executions': 'triggered_pipeline_executions',
        'user_history_as_transferer': 'user_history_as_transferer',
        'user_organization': 'user_organization',
        'user_starred_projects': 'user_starred_projects',
        'withdraw_history_organizations': 'withdraw_history_organizations',
        'withdraw_history_workloads': 'withdraw_history_workloads',
        'workspaces': 'workspaces'
    }

    def __init__(self, access_tokens=None, cluster_quotas=None, coupon_redemptions=None, created_volume_v2s=None, credit_earn_histories=None, default_organization=None, experiments=None, hub_model_task_histories=None, hub_model_tasks=None, judged_pipeline_step_executions=None, llm_api_keys=None, llm_documents=None, llm_knowledge=None, llm_knowledge_jobs=None, llm_model_connections=None, llm_model_parameter_definitions=None, llm_model_parameter_presets=None, llm_user_groups=None, llm_vectordb_connections=None, llm_workflow_revisions=None, llm_workflows=None, memberships=None, model_service_revisions=None, model_service_rollouts=None, model_services=None, models=None, onboardings=None, owned_organizations=None, persistent_artifacts=None, pipelines=None, project_dashboards=None, reset_password_tokens=None, run_executions=None, run_tracking_dashboards=None, starred_dashboards=None, triggered_pipeline_executions=None, user_history_as_transferer=None, user_organization=None, user_starred_projects=None, withdraw_history_organizations=None, withdraw_history_workloads=None, workspaces=None, local_vars_configuration=None):  # noqa: E501
        """OrmUserEdges - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._access_tokens = None
        self._cluster_quotas = None
        self._coupon_redemptions = None
        self._created_volume_v2s = None
        self._credit_earn_histories = None
        self._default_organization = None
        self._experiments = None
        self._hub_model_task_histories = None
        self._hub_model_tasks = None
        self._judged_pipeline_step_executions = None
        self._llm_api_keys = None
        self._llm_documents = None
        self._llm_knowledge = None
        self._llm_knowledge_jobs = None
        self._llm_model_connections = None
        self._llm_model_parameter_definitions = None
        self._llm_model_parameter_presets = None
        self._llm_user_groups = None
        self._llm_vectordb_connections = None
        self._llm_workflow_revisions = None
        self._llm_workflows = None
        self._memberships = None
        self._model_service_revisions = None
        self._model_service_rollouts = None
        self._model_services = None
        self._models = None
        self._onboardings = None
        self._owned_organizations = None
        self._persistent_artifacts = None
        self._pipelines = None
        self._project_dashboards = None
        self._reset_password_tokens = None
        self._run_executions = None
        self._run_tracking_dashboards = None
        self._starred_dashboards = None
        self._triggered_pipeline_executions = None
        self._user_history_as_transferer = None
        self._user_organization = None
        self._user_starred_projects = None
        self._withdraw_history_organizations = None
        self._withdraw_history_workloads = None
        self._workspaces = None
        self.discriminator = None

        if access_tokens is not None:
            self.access_tokens = access_tokens
        if cluster_quotas is not None:
            self.cluster_quotas = cluster_quotas
        if coupon_redemptions is not None:
            self.coupon_redemptions = coupon_redemptions
        if created_volume_v2s is not None:
            self.created_volume_v2s = created_volume_v2s
        if credit_earn_histories is not None:
            self.credit_earn_histories = credit_earn_histories
        if default_organization is not None:
            self.default_organization = default_organization
        if experiments is not None:
            self.experiments = experiments
        if hub_model_task_histories is not None:
            self.hub_model_task_histories = hub_model_task_histories
        if hub_model_tasks is not None:
            self.hub_model_tasks = hub_model_tasks
        if judged_pipeline_step_executions is not None:
            self.judged_pipeline_step_executions = judged_pipeline_step_executions
        if llm_api_keys is not None:
            self.llm_api_keys = llm_api_keys
        if llm_documents is not None:
            self.llm_documents = llm_documents
        if llm_knowledge is not None:
            self.llm_knowledge = llm_knowledge
        if llm_knowledge_jobs is not None:
            self.llm_knowledge_jobs = llm_knowledge_jobs
        if llm_model_connections is not None:
            self.llm_model_connections = llm_model_connections
        if llm_model_parameter_definitions is not None:
            self.llm_model_parameter_definitions = llm_model_parameter_definitions
        if llm_model_parameter_presets is not None:
            self.llm_model_parameter_presets = llm_model_parameter_presets
        if llm_user_groups is not None:
            self.llm_user_groups = llm_user_groups
        if llm_vectordb_connections is not None:
            self.llm_vectordb_connections = llm_vectordb_connections
        if llm_workflow_revisions is not None:
            self.llm_workflow_revisions = llm_workflow_revisions
        if llm_workflows is not None:
            self.llm_workflows = llm_workflows
        if memberships is not None:
            self.memberships = memberships
        if model_service_revisions is not None:
            self.model_service_revisions = model_service_revisions
        if model_service_rollouts is not None:
            self.model_service_rollouts = model_service_rollouts
        if model_services is not None:
            self.model_services = model_services
        if models is not None:
            self.models = models
        if onboardings is not None:
            self.onboardings = onboardings
        if owned_organizations is not None:
            self.owned_organizations = owned_organizations
        if persistent_artifacts is not None:
            self.persistent_artifacts = persistent_artifacts
        if pipelines is not None:
            self.pipelines = pipelines
        if project_dashboards is not None:
            self.project_dashboards = project_dashboards
        if reset_password_tokens is not None:
            self.reset_password_tokens = reset_password_tokens
        if run_executions is not None:
            self.run_executions = run_executions
        if run_tracking_dashboards is not None:
            self.run_tracking_dashboards = run_tracking_dashboards
        if starred_dashboards is not None:
            self.starred_dashboards = starred_dashboards
        if triggered_pipeline_executions is not None:
            self.triggered_pipeline_executions = triggered_pipeline_executions
        if user_history_as_transferer is not None:
            self.user_history_as_transferer = user_history_as_transferer
        if user_organization is not None:
            self.user_organization = user_organization
        if user_starred_projects is not None:
            self.user_starred_projects = user_starred_projects
        if withdraw_history_organizations is not None:
            self.withdraw_history_organizations = withdraw_history_organizations
        if withdraw_history_workloads is not None:
            self.withdraw_history_workloads = withdraw_history_workloads
        if workspaces is not None:
            self.workspaces = workspaces

    @property
    def access_tokens(self):
        """Gets the access_tokens of this OrmUserEdges.  # noqa: E501


        :return: The access_tokens of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmAccessToken]
        """
        return self._access_tokens

    @access_tokens.setter
    def access_tokens(self, access_tokens):
        """Sets the access_tokens of this OrmUserEdges.


        :param access_tokens: The access_tokens of this OrmUserEdges.  # noqa: E501
        :type access_tokens: list[OrmAccessToken]
        """

        self._access_tokens = access_tokens

    @property
    def cluster_quotas(self):
        """Gets the cluster_quotas of this OrmUserEdges.  # noqa: E501


        :return: The cluster_quotas of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmClusterQuota]
        """
        return self._cluster_quotas

    @cluster_quotas.setter
    def cluster_quotas(self, cluster_quotas):
        """Sets the cluster_quotas of this OrmUserEdges.


        :param cluster_quotas: The cluster_quotas of this OrmUserEdges.  # noqa: E501
        :type cluster_quotas: list[OrmClusterQuota]
        """

        self._cluster_quotas = cluster_quotas

    @property
    def coupon_redemptions(self):
        """Gets the coupon_redemptions of this OrmUserEdges.  # noqa: E501


        :return: The coupon_redemptions of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmCouponRedemption]
        """
        return self._coupon_redemptions

    @coupon_redemptions.setter
    def coupon_redemptions(self, coupon_redemptions):
        """Sets the coupon_redemptions of this OrmUserEdges.


        :param coupon_redemptions: The coupon_redemptions of this OrmUserEdges.  # noqa: E501
        :type coupon_redemptions: list[OrmCouponRedemption]
        """

        self._coupon_redemptions = coupon_redemptions

    @property
    def created_volume_v2s(self):
        """Gets the created_volume_v2s of this OrmUserEdges.  # noqa: E501


        :return: The created_volume_v2s of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmVolumeV2]
        """
        return self._created_volume_v2s

    @created_volume_v2s.setter
    def created_volume_v2s(self, created_volume_v2s):
        """Sets the created_volume_v2s of this OrmUserEdges.


        :param created_volume_v2s: The created_volume_v2s of this OrmUserEdges.  # noqa: E501
        :type created_volume_v2s: list[OrmVolumeV2]
        """

        self._created_volume_v2s = created_volume_v2s

    @property
    def credit_earn_histories(self):
        """Gets the credit_earn_histories of this OrmUserEdges.  # noqa: E501


        :return: The credit_earn_histories of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmCreditEarnHistory]
        """
        return self._credit_earn_histories

    @credit_earn_histories.setter
    def credit_earn_histories(self, credit_earn_histories):
        """Sets the credit_earn_histories of this OrmUserEdges.


        :param credit_earn_histories: The credit_earn_histories of this OrmUserEdges.  # noqa: E501
        :type credit_earn_histories: list[OrmCreditEarnHistory]
        """

        self._credit_earn_histories = credit_earn_histories

    @property
    def default_organization(self):
        """Gets the default_organization of this OrmUserEdges.  # noqa: E501


        :return: The default_organization of this OrmUserEdges.  # noqa: E501
        :rtype: OrmOrganization
        """
        return self._default_organization

    @default_organization.setter
    def default_organization(self, default_organization):
        """Sets the default_organization of this OrmUserEdges.


        :param default_organization: The default_organization of this OrmUserEdges.  # noqa: E501
        :type default_organization: OrmOrganization
        """

        self._default_organization = default_organization

    @property
    def experiments(self):
        """Gets the experiments of this OrmUserEdges.  # noqa: E501


        :return: The experiments of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmExperiment]
        """
        return self._experiments

    @experiments.setter
    def experiments(self, experiments):
        """Sets the experiments of this OrmUserEdges.


        :param experiments: The experiments of this OrmUserEdges.  # noqa: E501
        :type experiments: list[OrmExperiment]
        """

        self._experiments = experiments

    @property
    def hub_model_task_histories(self):
        """Gets the hub_model_task_histories of this OrmUserEdges.  # noqa: E501


        :return: The hub_model_task_histories of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmHubModelTaskHistory]
        """
        return self._hub_model_task_histories

    @hub_model_task_histories.setter
    def hub_model_task_histories(self, hub_model_task_histories):
        """Sets the hub_model_task_histories of this OrmUserEdges.


        :param hub_model_task_histories: The hub_model_task_histories of this OrmUserEdges.  # noqa: E501
        :type hub_model_task_histories: list[OrmHubModelTaskHistory]
        """

        self._hub_model_task_histories = hub_model_task_histories

    @property
    def hub_model_tasks(self):
        """Gets the hub_model_tasks of this OrmUserEdges.  # noqa: E501


        :return: The hub_model_tasks of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmHubModelTask]
        """
        return self._hub_model_tasks

    @hub_model_tasks.setter
    def hub_model_tasks(self, hub_model_tasks):
        """Sets the hub_model_tasks of this OrmUserEdges.


        :param hub_model_tasks: The hub_model_tasks of this OrmUserEdges.  # noqa: E501
        :type hub_model_tasks: list[OrmHubModelTask]
        """

        self._hub_model_tasks = hub_model_tasks

    @property
    def judged_pipeline_step_executions(self):
        """Gets the judged_pipeline_step_executions of this OrmUserEdges.  # noqa: E501


        :return: The judged_pipeline_step_executions of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmPipelineStepExecution]
        """
        return self._judged_pipeline_step_executions

    @judged_pipeline_step_executions.setter
    def judged_pipeline_step_executions(self, judged_pipeline_step_executions):
        """Sets the judged_pipeline_step_executions of this OrmUserEdges.


        :param judged_pipeline_step_executions: The judged_pipeline_step_executions of this OrmUserEdges.  # noqa: E501
        :type judged_pipeline_step_executions: list[OrmPipelineStepExecution]
        """

        self._judged_pipeline_step_executions = judged_pipeline_step_executions

    @property
    def llm_api_keys(self):
        """Gets the llm_api_keys of this OrmUserEdges.  # noqa: E501


        :return: The llm_api_keys of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmLLMApiKey]
        """
        return self._llm_api_keys

    @llm_api_keys.setter
    def llm_api_keys(self, llm_api_keys):
        """Sets the llm_api_keys of this OrmUserEdges.


        :param llm_api_keys: The llm_api_keys of this OrmUserEdges.  # noqa: E501
        :type llm_api_keys: list[OrmLLMApiKey]
        """

        self._llm_api_keys = llm_api_keys

    @property
    def llm_documents(self):
        """Gets the llm_documents of this OrmUserEdges.  # noqa: E501


        :return: The llm_documents of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmLLMDocument]
        """
        return self._llm_documents

    @llm_documents.setter
    def llm_documents(self, llm_documents):
        """Sets the llm_documents of this OrmUserEdges.


        :param llm_documents: The llm_documents of this OrmUserEdges.  # noqa: E501
        :type llm_documents: list[OrmLLMDocument]
        """

        self._llm_documents = llm_documents

    @property
    def llm_knowledge(self):
        """Gets the llm_knowledge of this OrmUserEdges.  # noqa: E501


        :return: The llm_knowledge of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmLLMKnowledge]
        """
        return self._llm_knowledge

    @llm_knowledge.setter
    def llm_knowledge(self, llm_knowledge):
        """Sets the llm_knowledge of this OrmUserEdges.


        :param llm_knowledge: The llm_knowledge of this OrmUserEdges.  # noqa: E501
        :type llm_knowledge: list[OrmLLMKnowledge]
        """

        self._llm_knowledge = llm_knowledge

    @property
    def llm_knowledge_jobs(self):
        """Gets the llm_knowledge_jobs of this OrmUserEdges.  # noqa: E501


        :return: The llm_knowledge_jobs of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmLLMKnowledgeJob]
        """
        return self._llm_knowledge_jobs

    @llm_knowledge_jobs.setter
    def llm_knowledge_jobs(self, llm_knowledge_jobs):
        """Sets the llm_knowledge_jobs of this OrmUserEdges.


        :param llm_knowledge_jobs: The llm_knowledge_jobs of this OrmUserEdges.  # noqa: E501
        :type llm_knowledge_jobs: list[OrmLLMKnowledgeJob]
        """

        self._llm_knowledge_jobs = llm_knowledge_jobs

    @property
    def llm_model_connections(self):
        """Gets the llm_model_connections of this OrmUserEdges.  # noqa: E501


        :return: The llm_model_connections of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmLLMModelConnection]
        """
        return self._llm_model_connections

    @llm_model_connections.setter
    def llm_model_connections(self, llm_model_connections):
        """Sets the llm_model_connections of this OrmUserEdges.


        :param llm_model_connections: The llm_model_connections of this OrmUserEdges.  # noqa: E501
        :type llm_model_connections: list[OrmLLMModelConnection]
        """

        self._llm_model_connections = llm_model_connections

    @property
    def llm_model_parameter_definitions(self):
        """Gets the llm_model_parameter_definitions of this OrmUserEdges.  # noqa: E501


        :return: The llm_model_parameter_definitions of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmLLMModelParameterDefinition]
        """
        return self._llm_model_parameter_definitions

    @llm_model_parameter_definitions.setter
    def llm_model_parameter_definitions(self, llm_model_parameter_definitions):
        """Sets the llm_model_parameter_definitions of this OrmUserEdges.


        :param llm_model_parameter_definitions: The llm_model_parameter_definitions of this OrmUserEdges.  # noqa: E501
        :type llm_model_parameter_definitions: list[OrmLLMModelParameterDefinition]
        """

        self._llm_model_parameter_definitions = llm_model_parameter_definitions

    @property
    def llm_model_parameter_presets(self):
        """Gets the llm_model_parameter_presets of this OrmUserEdges.  # noqa: E501


        :return: The llm_model_parameter_presets of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmLLMModelParameterPreset]
        """
        return self._llm_model_parameter_presets

    @llm_model_parameter_presets.setter
    def llm_model_parameter_presets(self, llm_model_parameter_presets):
        """Sets the llm_model_parameter_presets of this OrmUserEdges.


        :param llm_model_parameter_presets: The llm_model_parameter_presets of this OrmUserEdges.  # noqa: E501
        :type llm_model_parameter_presets: list[OrmLLMModelParameterPreset]
        """

        self._llm_model_parameter_presets = llm_model_parameter_presets

    @property
    def llm_user_groups(self):
        """Gets the llm_user_groups of this OrmUserEdges.  # noqa: E501


        :return: The llm_user_groups of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmLLMUserGroup]
        """
        return self._llm_user_groups

    @llm_user_groups.setter
    def llm_user_groups(self, llm_user_groups):
        """Sets the llm_user_groups of this OrmUserEdges.


        :param llm_user_groups: The llm_user_groups of this OrmUserEdges.  # noqa: E501
        :type llm_user_groups: list[OrmLLMUserGroup]
        """

        self._llm_user_groups = llm_user_groups

    @property
    def llm_vectordb_connections(self):
        """Gets the llm_vectordb_connections of this OrmUserEdges.  # noqa: E501


        :return: The llm_vectordb_connections of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmLLMVectorDBConnection]
        """
        return self._llm_vectordb_connections

    @llm_vectordb_connections.setter
    def llm_vectordb_connections(self, llm_vectordb_connections):
        """Sets the llm_vectordb_connections of this OrmUserEdges.


        :param llm_vectordb_connections: The llm_vectordb_connections of this OrmUserEdges.  # noqa: E501
        :type llm_vectordb_connections: list[OrmLLMVectorDBConnection]
        """

        self._llm_vectordb_connections = llm_vectordb_connections

    @property
    def llm_workflow_revisions(self):
        """Gets the llm_workflow_revisions of this OrmUserEdges.  # noqa: E501


        :return: The llm_workflow_revisions of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmLLMWorkflowRevision]
        """
        return self._llm_workflow_revisions

    @llm_workflow_revisions.setter
    def llm_workflow_revisions(self, llm_workflow_revisions):
        """Sets the llm_workflow_revisions of this OrmUserEdges.


        :param llm_workflow_revisions: The llm_workflow_revisions of this OrmUserEdges.  # noqa: E501
        :type llm_workflow_revisions: list[OrmLLMWorkflowRevision]
        """

        self._llm_workflow_revisions = llm_workflow_revisions

    @property
    def llm_workflows(self):
        """Gets the llm_workflows of this OrmUserEdges.  # noqa: E501


        :return: The llm_workflows of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmLLMWorkflow]
        """
        return self._llm_workflows

    @llm_workflows.setter
    def llm_workflows(self, llm_workflows):
        """Sets the llm_workflows of this OrmUserEdges.


        :param llm_workflows: The llm_workflows of this OrmUserEdges.  # noqa: E501
        :type llm_workflows: list[OrmLLMWorkflow]
        """

        self._llm_workflows = llm_workflows

    @property
    def memberships(self):
        """Gets the memberships of this OrmUserEdges.  # noqa: E501


        :return: The memberships of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmUserGroupMembership]
        """
        return self._memberships

    @memberships.setter
    def memberships(self, memberships):
        """Sets the memberships of this OrmUserEdges.


        :param memberships: The memberships of this OrmUserEdges.  # noqa: E501
        :type memberships: list[OrmUserGroupMembership]
        """

        self._memberships = memberships

    @property
    def model_service_revisions(self):
        """Gets the model_service_revisions of this OrmUserEdges.  # noqa: E501


        :return: The model_service_revisions of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmModelServiceRevision]
        """
        return self._model_service_revisions

    @model_service_revisions.setter
    def model_service_revisions(self, model_service_revisions):
        """Sets the model_service_revisions of this OrmUserEdges.


        :param model_service_revisions: The model_service_revisions of this OrmUserEdges.  # noqa: E501
        :type model_service_revisions: list[OrmModelServiceRevision]
        """

        self._model_service_revisions = model_service_revisions

    @property
    def model_service_rollouts(self):
        """Gets the model_service_rollouts of this OrmUserEdges.  # noqa: E501


        :return: The model_service_rollouts of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmModelServiceRollout]
        """
        return self._model_service_rollouts

    @model_service_rollouts.setter
    def model_service_rollouts(self, model_service_rollouts):
        """Sets the model_service_rollouts of this OrmUserEdges.


        :param model_service_rollouts: The model_service_rollouts of this OrmUserEdges.  # noqa: E501
        :type model_service_rollouts: list[OrmModelServiceRollout]
        """

        self._model_service_rollouts = model_service_rollouts

    @property
    def model_services(self):
        """Gets the model_services of this OrmUserEdges.  # noqa: E501


        :return: The model_services of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmModelService]
        """
        return self._model_services

    @model_services.setter
    def model_services(self, model_services):
        """Sets the model_services of this OrmUserEdges.


        :param model_services: The model_services of this OrmUserEdges.  # noqa: E501
        :type model_services: list[OrmModelService]
        """

        self._model_services = model_services

    @property
    def models(self):
        """Gets the models of this OrmUserEdges.  # noqa: E501


        :return: The models of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmModel]
        """
        return self._models

    @models.setter
    def models(self, models):
        """Sets the models of this OrmUserEdges.


        :param models: The models of this OrmUserEdges.  # noqa: E501
        :type models: list[OrmModel]
        """

        self._models = models

    @property
    def onboardings(self):
        """Gets the onboardings of this OrmUserEdges.  # noqa: E501


        :return: The onboardings of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmOnboarding]
        """
        return self._onboardings

    @onboardings.setter
    def onboardings(self, onboardings):
        """Sets the onboardings of this OrmUserEdges.


        :param onboardings: The onboardings of this OrmUserEdges.  # noqa: E501
        :type onboardings: list[OrmOnboarding]
        """

        self._onboardings = onboardings

    @property
    def owned_organizations(self):
        """Gets the owned_organizations of this OrmUserEdges.  # noqa: E501


        :return: The owned_organizations of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmOrganization]
        """
        return self._owned_organizations

    @owned_organizations.setter
    def owned_organizations(self, owned_organizations):
        """Sets the owned_organizations of this OrmUserEdges.


        :param owned_organizations: The owned_organizations of this OrmUserEdges.  # noqa: E501
        :type owned_organizations: list[OrmOrganization]
        """

        self._owned_organizations = owned_organizations

    @property
    def persistent_artifacts(self):
        """Gets the persistent_artifacts of this OrmUserEdges.  # noqa: E501


        :return: The persistent_artifacts of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmArtifact]
        """
        return self._persistent_artifacts

    @persistent_artifacts.setter
    def persistent_artifacts(self, persistent_artifacts):
        """Sets the persistent_artifacts of this OrmUserEdges.


        :param persistent_artifacts: The persistent_artifacts of this OrmUserEdges.  # noqa: E501
        :type persistent_artifacts: list[OrmArtifact]
        """

        self._persistent_artifacts = persistent_artifacts

    @property
    def pipelines(self):
        """Gets the pipelines of this OrmUserEdges.  # noqa: E501


        :return: The pipelines of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmPipeline]
        """
        return self._pipelines

    @pipelines.setter
    def pipelines(self, pipelines):
        """Sets the pipelines of this OrmUserEdges.


        :param pipelines: The pipelines of this OrmUserEdges.  # noqa: E501
        :type pipelines: list[OrmPipeline]
        """

        self._pipelines = pipelines

    @property
    def project_dashboards(self):
        """Gets the project_dashboards of this OrmUserEdges.  # noqa: E501


        :return: The project_dashboards of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmProjectDashboard]
        """
        return self._project_dashboards

    @project_dashboards.setter
    def project_dashboards(self, project_dashboards):
        """Sets the project_dashboards of this OrmUserEdges.


        :param project_dashboards: The project_dashboards of this OrmUserEdges.  # noqa: E501
        :type project_dashboards: list[OrmProjectDashboard]
        """

        self._project_dashboards = project_dashboards

    @property
    def reset_password_tokens(self):
        """Gets the reset_password_tokens of this OrmUserEdges.  # noqa: E501


        :return: The reset_password_tokens of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmResetPasswordToken]
        """
        return self._reset_password_tokens

    @reset_password_tokens.setter
    def reset_password_tokens(self, reset_password_tokens):
        """Sets the reset_password_tokens of this OrmUserEdges.


        :param reset_password_tokens: The reset_password_tokens of this OrmUserEdges.  # noqa: E501
        :type reset_password_tokens: list[OrmResetPasswordToken]
        """

        self._reset_password_tokens = reset_password_tokens

    @property
    def run_executions(self):
        """Gets the run_executions of this OrmUserEdges.  # noqa: E501


        :return: The run_executions of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmRunExecution]
        """
        return self._run_executions

    @run_executions.setter
    def run_executions(self, run_executions):
        """Sets the run_executions of this OrmUserEdges.


        :param run_executions: The run_executions of this OrmUserEdges.  # noqa: E501
        :type run_executions: list[OrmRunExecution]
        """

        self._run_executions = run_executions

    @property
    def run_tracking_dashboards(self):
        """Gets the run_tracking_dashboards of this OrmUserEdges.  # noqa: E501


        :return: The run_tracking_dashboards of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmRunTrackingDashboard]
        """
        return self._run_tracking_dashboards

    @run_tracking_dashboards.setter
    def run_tracking_dashboards(self, run_tracking_dashboards):
        """Sets the run_tracking_dashboards of this OrmUserEdges.


        :param run_tracking_dashboards: The run_tracking_dashboards of this OrmUserEdges.  # noqa: E501
        :type run_tracking_dashboards: list[OrmRunTrackingDashboard]
        """

        self._run_tracking_dashboards = run_tracking_dashboards

    @property
    def starred_dashboards(self):
        """Gets the starred_dashboards of this OrmUserEdges.  # noqa: E501


        :return: The starred_dashboards of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmProjectDashboard]
        """
        return self._starred_dashboards

    @starred_dashboards.setter
    def starred_dashboards(self, starred_dashboards):
        """Sets the starred_dashboards of this OrmUserEdges.


        :param starred_dashboards: The starred_dashboards of this OrmUserEdges.  # noqa: E501
        :type starred_dashboards: list[OrmProjectDashboard]
        """

        self._starred_dashboards = starred_dashboards

    @property
    def triggered_pipeline_executions(self):
        """Gets the triggered_pipeline_executions of this OrmUserEdges.  # noqa: E501


        :return: The triggered_pipeline_executions of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmPipelineExecution]
        """
        return self._triggered_pipeline_executions

    @triggered_pipeline_executions.setter
    def triggered_pipeline_executions(self, triggered_pipeline_executions):
        """Sets the triggered_pipeline_executions of this OrmUserEdges.


        :param triggered_pipeline_executions: The triggered_pipeline_executions of this OrmUserEdges.  # noqa: E501
        :type triggered_pipeline_executions: list[OrmPipelineExecution]
        """

        self._triggered_pipeline_executions = triggered_pipeline_executions

    @property
    def user_history_as_transferer(self):
        """Gets the user_history_as_transferer of this OrmUserEdges.  # noqa: E501


        :return: The user_history_as_transferer of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmUserHistory]
        """
        return self._user_history_as_transferer

    @user_history_as_transferer.setter
    def user_history_as_transferer(self, user_history_as_transferer):
        """Sets the user_history_as_transferer of this OrmUserEdges.


        :param user_history_as_transferer: The user_history_as_transferer of this OrmUserEdges.  # noqa: E501
        :type user_history_as_transferer: list[OrmUserHistory]
        """

        self._user_history_as_transferer = user_history_as_transferer

    @property
    def user_organization(self):
        """Gets the user_organization of this OrmUserEdges.  # noqa: E501


        :return: The user_organization of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmUserOrganization]
        """
        return self._user_organization

    @user_organization.setter
    def user_organization(self, user_organization):
        """Sets the user_organization of this OrmUserEdges.


        :param user_organization: The user_organization of this OrmUserEdges.  # noqa: E501
        :type user_organization: list[OrmUserOrganization]
        """

        self._user_organization = user_organization

    @property
    def user_starred_projects(self):
        """Gets the user_starred_projects of this OrmUserEdges.  # noqa: E501


        :return: The user_starred_projects of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmUserStarredProjects]
        """
        return self._user_starred_projects

    @user_starred_projects.setter
    def user_starred_projects(self, user_starred_projects):
        """Sets the user_starred_projects of this OrmUserEdges.


        :param user_starred_projects: The user_starred_projects of this OrmUserEdges.  # noqa: E501
        :type user_starred_projects: list[OrmUserStarredProjects]
        """

        self._user_starred_projects = user_starred_projects

    @property
    def withdraw_history_organizations(self):
        """Gets the withdraw_history_organizations of this OrmUserEdges.  # noqa: E501


        :return: The withdraw_history_organizations of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmWithdrawHistoryOrganization]
        """
        return self._withdraw_history_organizations

    @withdraw_history_organizations.setter
    def withdraw_history_organizations(self, withdraw_history_organizations):
        """Sets the withdraw_history_organizations of this OrmUserEdges.


        :param withdraw_history_organizations: The withdraw_history_organizations of this OrmUserEdges.  # noqa: E501
        :type withdraw_history_organizations: list[OrmWithdrawHistoryOrganization]
        """

        self._withdraw_history_organizations = withdraw_history_organizations

    @property
    def withdraw_history_workloads(self):
        """Gets the withdraw_history_workloads of this OrmUserEdges.  # noqa: E501


        :return: The withdraw_history_workloads of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmWithdrawHistoryWorkload]
        """
        return self._withdraw_history_workloads

    @withdraw_history_workloads.setter
    def withdraw_history_workloads(self, withdraw_history_workloads):
        """Sets the withdraw_history_workloads of this OrmUserEdges.


        :param withdraw_history_workloads: The withdraw_history_workloads of this OrmUserEdges.  # noqa: E501
        :type withdraw_history_workloads: list[OrmWithdrawHistoryWorkload]
        """

        self._withdraw_history_workloads = withdraw_history_workloads

    @property
    def workspaces(self):
        """Gets the workspaces of this OrmUserEdges.  # noqa: E501


        :return: The workspaces of this OrmUserEdges.  # noqa: E501
        :rtype: list[OrmWorkspace]
        """
        return self._workspaces

    @workspaces.setter
    def workspaces(self, workspaces):
        """Sets the workspaces of this OrmUserEdges.


        :param workspaces: The workspaces of this OrmUserEdges.  # noqa: E501
        :type workspaces: list[OrmWorkspace]
        """

        self._workspaces = workspaces

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmUserEdges):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmUserEdges):
            return True

        return self.to_dict() != other.to_dict()



from typing import Optional

import click

from vessl.openapi_client import ResponseStorageV2Info
from vessl.cli._base import (
    VesslGroup,
    vessl_argument,
    vessl_conditional_option,
    vessl_option,
)
from vessl.cli._util import print_table, truncate_datetime
from vessl.storage import StorageType, create_storage, delete_storage, list_storages
from vessl.util.prompt import generic_prompter, prompt_confirm


@click.command(name="storage", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command()
@vessl_option(
    "--limit",
    type=click.INT,
    required=False,
    default=None,
)
def list(limit: Optional[int]):
    storages = list_storages(limit=limit)
    print_table(
        storages,
        ["Name", "IsVESSLManaged", "Path", "Created", "Updated"],
        lambda x: [
            x.name,
            x.is_vessl_managed,
            _get_storage_path(x),
            truncate_datetime(x.created_dt),
            truncate_datetime(x.updated_dt),
        ],
    )


def _get_storage_path(storage: ResponseStorageV2Info):
    if storage.is_vessl_managed:
        return ""
    if storage.type == StorageType.GCS.value:
        return f"gs://{storage.config.gcs_config.bucket_name}"
    if storage.type == StorageType.S3.value:
        return f"s3://{storage.config.s3_config.bucket_name}"
    if storage.type == StorageType.NFS.value:
        return f"nfs://{storage.config.cluster_nfs_config.host}:{storage.config.cluster_nfs_config.base_path}"
    if storage.type == StorageType.HOST_PATH.value:
        return f"host-path:{storage.config.cluster_host_path_config.base_path}"


def _storage_type_callback(ctx: click.Context, param: click.Parameter, value: str) -> StorageType:
    if value == "gcs":
        return StorageType.GCS
    if value == "s3":
        return StorageType.S3
    if value == "nfs":
        return StorageType.NFS
    if value == "host-path":
        return StorageType.HOST_PATH
    raise click.BadParameter("Unknown storage type")


@cli.vessl_command()
@vessl_argument(
    "name",
    type=click.STRING,
    required=True,
    prompter=generic_prompter("Storage name"),
)
@vessl_option(
    "--storage-type",
    type=click.Choice(["gcs", "s3", "nfs", "host-path"]),
    prompter=generic_prompter("Storage type; gcs|s3|nfs|host-path"),
    help="\n".join(
        [
            "\b",
            "Specify the storage type. Supported values:",
            "- `gcs`       : Google Cloud Storage",
            "- `s3`        : Amazon S3 storage",
            "- `nfs`       : NFS storage",
            "- `host-path` : host-based local storage",
        ]
    ),
    required=True,
    callback=_storage_type_callback,
)
@vessl_option(
    "--path",
    prompt="Storage path",
    prompter=generic_prompter("Storage path"),
    help="\n".join(
        [
            "\b",
            "Specify the path for the operation. The path format must match the selected storage type:",
            "- For `gcs`       : Path must in `{bucket_name}/{path}` (e.g. my-bucket)",
            "- For `s3`        : Path must in `{bucket_name}/{path}` (e.g. my-bucket)",
            "- For `nfs`       : Path must be in `{server}:{path}` format (e.g. 192.168.1.100:/shared/data)",
            "- For `host-path` : Path must be an absolute local path (e.g. /data/host-folder)",
        ]
    ),
    required=True,
)
@vessl_conditional_option(
    "--credential-name",
    prompt="Credentials name",
    prompter=generic_prompter("Credentials name"),
    condition=("storage_type", [StorageType.GCS.value, StorageType.S3.value]),
    help="Specify the credential name of the storage. This option is required when Storage Type (`--storage-type`) is gcs|s3.",
)
@vessl_conditional_option(
    "--cluster-name",
    prompt="Cluster name",
    condition=(
        "storage_type",
        [StorageType.NFS.value, StorageType.HOST_PATH.value],
    ),
    help="Specify the name of the cluster. This option is required when Storage type (`--storage-type`) is nfs|host-path",
)
def create(
    name: str,
    storage_type: StorageType,
    path: str,
    credential_name: Optional[str],
    cluster_name: Optional[str],
):
    create_storage(name, storage_type, path, credential_name, cluster_name)


@cli.vessl_command()
@vessl_argument(
    "name",
    type=click.STRING,
    required=True,
    prompter=generic_prompter("Storage name"),
)
def delete(name: str):
    if not prompt_confirm(f"Are you sure to delete external storage `{name}`?"):
        return

    delete_storage(name)



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class V1FormError(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'decode_errors': 'list[V1FormSingleError]',
        'not_injected_errors': 'list[V1NotInjectedError]',
        'validation_errors': 'list[V1FormSingleError]',
        'warnings': 'list[V1FormSingleError]'
    }

    attribute_map = {
        'decode_errors': 'DecodeErrors',
        'not_injected_errors': 'NotInjectedErrors',
        'validation_errors': 'ValidationErrors',
        'warnings': 'Warnings'
    }

    def __init__(self, decode_errors=None, not_injected_errors=None, validation_errors=None, warnings=None, local_vars_configuration=None):  # noqa: E501
        """V1FormError - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._decode_errors = None
        self._not_injected_errors = None
        self._validation_errors = None
        self._warnings = None
        self.discriminator = None

        if decode_errors is not None:
            self.decode_errors = decode_errors
        if not_injected_errors is not None:
            self.not_injected_errors = not_injected_errors
        if validation_errors is not None:
            self.validation_errors = validation_errors
        if warnings is not None:
            self.warnings = warnings

    @property
    def decode_errors(self):
        """Gets the decode_errors of this V1FormError.  # noqa: E501


        :return: The decode_errors of this V1FormError.  # noqa: E501
        :rtype: list[V1FormSingleError]
        """
        return self._decode_errors

    @decode_errors.setter
    def decode_errors(self, decode_errors):
        """Sets the decode_errors of this V1FormError.


        :param decode_errors: The decode_errors of this V1FormError.  # noqa: E501
        :type decode_errors: list[V1FormSingleError]
        """

        self._decode_errors = decode_errors

    @property
    def not_injected_errors(self):
        """Gets the not_injected_errors of this V1FormError.  # noqa: E501


        :return: The not_injected_errors of this V1FormError.  # noqa: E501
        :rtype: list[V1NotInjectedError]
        """
        return self._not_injected_errors

    @not_injected_errors.setter
    def not_injected_errors(self, not_injected_errors):
        """Sets the not_injected_errors of this V1FormError.


        :param not_injected_errors: The not_injected_errors of this V1FormError.  # noqa: E501
        :type not_injected_errors: list[V1NotInjectedError]
        """

        self._not_injected_errors = not_injected_errors

    @property
    def validation_errors(self):
        """Gets the validation_errors of this V1FormError.  # noqa: E501


        :return: The validation_errors of this V1FormError.  # noqa: E501
        :rtype: list[V1FormSingleError]
        """
        return self._validation_errors

    @validation_errors.setter
    def validation_errors(self, validation_errors):
        """Sets the validation_errors of this V1FormError.


        :param validation_errors: The validation_errors of this V1FormError.  # noqa: E501
        :type validation_errors: list[V1FormSingleError]
        """

        self._validation_errors = validation_errors

    @property
    def warnings(self):
        """Gets the warnings of this V1FormError.  # noqa: E501


        :return: The warnings of this V1FormError.  # noqa: E501
        :rtype: list[V1FormSingleError]
        """
        return self._warnings

    @warnings.setter
    def warnings(self, warnings):
        """Sets the warnings of this V1FormError.


        :param warnings: The warnings of this V1FormError.  # noqa: E501
        :type warnings: list[V1FormSingleError]
        """

        self._warnings = warnings

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1FormError):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1FormError):
            return True

        return self.to_dict() != other.to_dict()



import os
from typing import Optional, Union

import six

from vessl.util import logger
from vessl.util.common import generate_uuid, get_module
from vessl.util.constant import VESSL_MEDIA_PATH
from vessl.util.exception import InvalidParamsError


class Audio:
    def __init__(
        self,
        data_or_path: Union[str, "Audio"],
        sample_rate: int = None,
        caption: Optional[str] = None,
    ):
        self._audio = None
        self._sample_rate = None
        self._caption = None
        self._path = None

        self._init_audio(data_or_path)
        self._init_meta(sample_rate, caption)
        self._save_audio()

    def _init_meta(self, sample_rate, caption):
        self._sample_rate = sample_rate
        self._caption = caption

        os.makedirs(VESSL_MEDIA_PATH, exist_ok=True)
        self._path = os.path.join(VESSL_MEDIA_PATH, generate_uuid() + ".wav")

    def _init_audio_from_path(self, path):
        sf = get_module(
            "soundfile",
            required='soundfile package is required. Run "pip install soundfile".',
        )
        self._audio, self._sample_rate = sf.read(path)

    def _init_audio_from_data(self, data):
        if self._sample_rate is None:
            raise InvalidParamsError("sample_rate is required for Vessl Audio class")
        self._audio = data

    def _init_audio(self, data_or_path):
        if isinstance(data_or_path, Audio):
            self._audio = data_or_path._audio
        elif isinstance(data_or_path, six.string_types):
            self._init_audio_from_path(data_or_path)
        else:
            self._init_audio_from_data(data_or_path)

    def _save_audio(self):
        sf = get_module(
            "soundfile",
            required='Soundfile package is required. Run "pip install soundfile".',
        )
        sf.write(self._path, self._audio, self._sample_rate)

    def flush(self):
        if os.path.isfile(self._path):
            os.remove(self._path)
        else:
            logger.error(f"Error: {self._path} file not found")

    @property
    def path(self):
        return self._path

    @property
    def caption(self):
        return self._caption



import os
from typing import List, Optional

from vessl.openapi_client import DistributedExperimentCreateAPIInput
from vessl.openapi_client.models import (
    ExperimentCreateAPIInput,
    InfluxdbWorkloadLog,
    OrmHyperparameter,
    ResponseExperimentInfo,
    ResponseExperimentListResponse,
    StorageFile,
)
from vessl import vessl_api
from vessl.kernel_cluster import read_cluster
from vessl.kernel_resource_spec import (
    _configure_custom_kernel_resource_spec,
    read_kernel_resource_spec,
)
from vessl.organization import _get_organization_name
from vessl.project import _get_project_name
from vessl.util import logger
from vessl.util.common import safe_cast
from vessl.util.constant import MOUNT_PATH_OUTPUT, SOURCE_TYPE_OUTPUT
from vessl.util.exception import (
    InvalidParamsError,
    VesslApiException,
)
from vessl.volume import (
    _configure_volume_mount_requests,
    copy_volume_file,
    list_volume_files,
)


def read_experiment(
    experiment_number: int,
    **kwargs,
) -> ResponseExperimentInfo:
    """Read experiment in the default organization/project. If you want to
    override the default organization/project, then pass `organization_name` or
    `project_name` as `**kwargs`.

    Args:
        experiment_number(int): experiment number.

    Example:
        ```python
        vessl.read_experiment(
            experiment_number=23,
        )
        ```
    """
    return vessl_api.experiment_read_api(
        experiment_number=experiment_number,
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
    )


def read_experiment_by_id(experiment_id: int) -> ResponseExperimentInfo:
    result = vessl_api.experiment_read_by_idapi(experiment_id=experiment_id)
    assert isinstance(result, ResponseExperimentInfo)
    return result


def update_experiment(experiment_number: int, message: str, **kwargs):
    """Update experiment in the default organization/project. If you want to
    override the default organization/project, then pass `organization_name` or
    `project_name` as `**kwargs`.

    Args:
        experiment_number(int): experiment number.
        message(str): message of experiment to update.

    Example:
        ```python
        vessl.update_experiment(
            experiment_number=23,
            message="Update # of hidden layer 32->64",
        )
        ```
    """
    return vessl_api.experiment_update_api(
        experiment_number=experiment_number,
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
        experiment_update_api_input={"message": message},
    )


def list_experiments(
    statuses: List[str] = None,
    **kwargs,
) -> List[ResponseExperimentListResponse]:
    """List experiments in the default organization/project. If you want to
    override the default organization/project, then pass `organization_name` or
    `project_name` as `**kwargs`.

    Args:
        statuses(List[str]): A list of status filter. Defaults to None.

    Example:
        ```python
        vessl.list_experiments(
            statuses=["completed"]
        )
        ```
    """
    statuses = (
        [",".join(statuses)] if statuses else None
    )  # since openapi-generator uses repeating params instead of commas

    query_keys = set(["limit", "offset"])
    query_kwargs = {k: v for k, v in kwargs.items() if k in query_keys}

    return vessl_api.experiment_list_api(
        statuses=statuses,
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
        **query_kwargs,
    ).results


def create_experiment(
    cluster_name: str,
    start_command: str,
    cluster_node_names: List[str] = None,
    kernel_resource_spec_name: str = None,
    processor_type: str = None,
    cpu_limit: float = None,
    memory_limit: str = None,
    gpu_type: str = None,
    gpu_limit: int = None,
    kernel_image_url: str = None,
    docker_credentials_id: Optional[int] = None,
    *,
    message: str = None,
    termination_protection: bool = False,
    hyperparameters: List[str] = None,
    secrets: List[str] = None,
    dataset_mounts: List[str] = None,
    model_mounts: List[str] = None,
    git_ref_mounts: List[str] = None,
    git_diff_mount: str = None,
    local_files: List[str] = None,
    use_vesslignore: bool = True,
    upload_local_git_diff: bool = False,
    archive_file_mount: str = None,
    object_storage_mounts: List[str] = None,
    root_volume_size: str = None,
    working_dir: str = None,
    output_dir: str = MOUNT_PATH_OUTPUT,
    worker_count: int = 1,
    framework_type: str = None,
    service_account: str = "",
    **kwargs,
) -> ResponseExperimentInfo:
    """Create experiment in the default organization/project. If you want to
    override the default organization/project, then pass `organization_name` or
    `project_name` as `**kwargs`. You can also configure git info by passing
    `git_branch` or `git_ref` as `**kwargs`. Pass `use_git_diff=True` if
    you want to run experiment with uncommitted changes and pass
    `use_git_diff_untracked=True` if you want to run untracked changes(only
    valid if `use_git_diff` is set).

    Args:
        cluster_name(str): Cluster name(must be specified before other options).
        cluster_node_names(List[str]): Node names. The experiment will run on
            one of these nodes. Defaults to None(all).
        start_command(str): Start command to execute in experiment container.
        kernel_resource_spec_name(str):  Resource type to run an experiment (for
            managed cluster only). Defaults to None.
        processor_type(str) cpu or gpu (for custom cluster only). Defaults to
            None.
        cpu_limit(float): Number of vCPUs (for custom cluster only). Defaults to
            None.
        memory_limit(str): Memory limit in GiB (for custom cluster only).
            Defaults to None.
        gpu_type(str): GPU type (for custom cluster only). Defaults to None.
        gpu_limit(int): Number of GPU cores (for custom cluster only). Defaults
            to None.
        kernel_image_url(str): Kernel docker image URL. Defaults to None.
        docker_credentials_id(int): Docker credential id. Defaults to None.
        message(str): Message. Defaults to None.
        termination_protection(bool): True if termination protection is enabled,
            False otherwise. Defaults to False.
        hyperparameters(List[str]): A list of hyperparameters. Defaults to None.
        secrets(List[str]): A list of secrets in form "KEY=VALUE". Defaults to None.
        dataset_mounts(List[str]): A list of dataset mounts. Defaults to None.
        model_mounts(List[str]): A list of model mounts. Defaults to None.
        git_ref_mounts(List[str]): A list of git repository mounts. Defaults to
            None.
        git_diff_mount(str): Git diff mounts. Defaults to None.
        local_files(List[str]): A list of local files to upload. Defaults to
            None.
        use_vesslignore(bool): True if local files matching glob patterns
            in .vesslignore files should be ignored. Patterns apply relative to
            the directory containing that .vesslignore file.
        upload_local_git_diff(bool): True if local git diff to upload, False
            otherwise. Defaults to False.
        archive_file_mount(str): Local archive file mounts. Defaults to None.
        object_storage_mounts(List[str]): Object storage mounts. Defaults to None.
        root_volume_size(str): Root volume size. Defaults to None.
        working_dir(str): Working directory path. Defaults to None.
        output_dir(str): Output directory path. Defaults to "/output/".
        worker_count(int): Number of workers(for distributed experiment only).
            Defaults to 1.
        framework_type(str): Specify "pytorch" or "tensorflow" (for distributed
            experiment only). Defaults to None.
        service_account(str): Service account name. Defaults to "".

    Example:
        ```python
        vessl.create_experiment(
            cluster_name="aws-apne2",
            kernel_resource_spec_name="v1.cpu-4.mem-13",
            kernel_image_url="public.ecr.aws/vessl/kernels:py36.full-cpu",
            dataset_mounts=["/input/:mnist"]
            start_command="pip install requirements.txt && python main.py",
        )
        ```
    """
    cluster = read_cluster(cluster_name)

    # Only allow service_account when the cluster is user-managed
    if cluster.is_vessl_managed and service_account:
        raise InvalidParamsError(
            "--service-account: only supported when running experiment in user-managed clusters"
        )

    kernel_resource_spec = kernel_resource_spec_id = None
    if kernel_resource_spec_name:
        kernel_resource_spec_id = read_kernel_resource_spec(
            cluster.id,
            kernel_resource_spec_name,
        ).id
    else:
        kernel_resource_spec = _configure_custom_kernel_resource_spec(
            processor_type,
            cpu_limit,
            memory_limit,
            gpu_type,
            gpu_limit,
        )

    volume_mount_requests = _configure_volume_mount_requests(
        dataset_mounts=dataset_mounts,
        git_ref_mounts=git_ref_mounts,
        git_diff_mount=git_diff_mount,
        archive_file_mount=archive_file_mount,
        object_storage_mounts=object_storage_mounts,
        root_volume_size=root_volume_size,
        working_dir=working_dir,
        output_dir=output_dir,
        model_mounts=model_mounts,
        local_files=local_files,
        use_vesslignore=use_vesslignore,
        upload_local_git_diff=upload_local_git_diff,
        **kwargs,
    )
    if worker_count > 1:
        return vessl_api.distributed_experiment_create_api(
            organization_name=_get_organization_name(**kwargs),
            project_name=_get_project_name(**kwargs),
            distributed_experiment_create_api_input=DistributedExperimentCreateAPIInput(
                cluster_id=cluster.id,
                hyperparameters=[
                    OrmHyperparameter(key=key, value=str(value))
                    for key, value in map(
                        lambda hyperparameter: hyperparameter.split("="),
                        (hyperparameters or []),
                    )
                ],
                framework_type=framework_type,
                image_url=kernel_image_url,
                docker_credentials_id=docker_credentials_id,
                message=message,
                worker_replicas=worker_count,
                worker_resource_spec=kernel_resource_spec,
                worker_resource_spec_id=kernel_resource_spec_id,
                start_command=start_command,
                termination_protection=termination_protection,
                volumes=volume_mount_requests,
                service_account_name=service_account,
            ),
        )

    organization_name = _get_organization_name(**kwargs)
    cluster_node_ids = None
    if not cluster.is_savvihub_managed and cluster_node_names:
        nodes = vessl_api.custom_cluster_node_list_api(
            organization_name=organization_name,
            cluster_id=cluster.id,
        ).nodes
        cluster_node_ids = [node.id for node in nodes if node.name in cluster_node_names]

    all_hyperparameters: List[OrmHyperparameter] = [
        OrmHyperparameter(key=key, value=str(value))
        for key, value in map(
            lambda hyperparameter: hyperparameter.split("="),
            (hyperparameters or []),
        )
    ] + [
        OrmHyperparameter(key=key, value=str(value), secret=True)
        for key, value in map(
            lambda secret: secret.split("="),
            (secrets or []),
        )
    ]

    return vessl_api.experiment_create_api(
        organization_name=organization_name,
        project_name=_get_project_name(**kwargs),
        experiment_create_api_input=ExperimentCreateAPIInput(
            cluster_id=cluster.id,
            cluster_node_ids=cluster_node_ids,
            hyperparameters=all_hyperparameters,
            image_url=kernel_image_url,
            docker_credentials_id=docker_credentials_id,
            message=message,
            resource_spec=kernel_resource_spec,
            resource_spec_id=kernel_resource_spec_id,
            start_command=start_command,
            termination_protection=termination_protection,
            volumes=volume_mount_requests,
            service_account_name=service_account,
        ),
    )


def list_experiment_logs(
    experiment_number: int,
    tail: int = 200,
    worker_number: int = 0,
    after: int = 0,
    **kwargs,
) -> List[InfluxdbWorkloadLog]:
    """List experiment logs in the default organization/project. If you want to
    override the default organization/project, then pass `organization_name` or
    `project_name` as `**kwargs`.

    Args:
        experiment_name (int): Experiment number.
        tail (int): The number of lines to display from the end. Display all if
            -1. Defaults to 200.
        worker_number (int): Override default worker number (for distributed
            experiments only). Defaults to 0.
        after (int): The number of starting lines to display from the start.
            Defaults to 0.

    Example:
        ```python
        vessl.list_experiment_logs(
            experiment_number=23,
        )
        ```
    """
    if tail == -1:
        tail = None

    experiment = read_experiment(experiment_number, **kwargs)

    if experiment.is_distributed:
        str_worker_number = safe_cast(worker_number, str, "0")
        return vessl_api.distributed_experiment_logs_api(
            experiment_number=experiment_number,
            limit=tail,
            organization_name=_get_organization_name(**kwargs),
            project_name=_get_project_name(**kwargs),
            distributed_number=worker_number,
        ).logs[str_worker_number]

    return vessl_api.experiment_logs_api(
        experiment_number=experiment_number,
        limit=tail,
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
        after=after,
    ).logs


def list_experiment_output_files(
    experiment_number: int,
    need_download_url: bool = False,
    recursive: bool = True,
    worker_number: int = 0,
    **kwargs,
) -> List[StorageFile]:
    """List experiment output files in the default organization/project. If you
    want to override the default organization/project, then pass
    `organization_name` or `project_name` as `**kwargs`.

    Args:
        experiment_number(int): Experiment number.
        need_download_url(bool): True if you need a download URL, False
            otherwise. Defaults to False.
        recursive(bool): True if list files recursively, False otherwise.
            Defaults to True.
        worker_number(int): Override default worker number (for distributed
            experiments only). Defaults to 0.

    Example:
        ```python
        vessl.list_experiment_output_files(
            experiment_number=23,
        )
        ```
    """
    experiment = read_experiment(experiment_number, **kwargs)

    if experiment.is_local:
        volume_id = experiment.local_execution_spec.output_volume_id
        return list_volume_files(
            volume_id=volume_id,
            need_download_url=need_download_url,
            path="",
            recursive=recursive,
        )

    if experiment.is_distributed:
        worker_replicas = experiment.distributed_spec.pytorch_spec.worker_replicas
        if worker_number >= worker_replicas:
            raise InvalidParamsError(
                "worker-number: should be less than {}".format(worker_replicas)
            )

        mounts = experiment.volume_mounts_list[worker_number].mounts
    else:
        mounts = experiment.volume_mounts.mounts

    for volume_mount in mounts:
        if volume_mount.source_type == SOURCE_TYPE_OUTPUT:
            base_path = volume_mount.volume.sub_path
            try:
                files = list_volume_files(
                    volume_id=volume_mount.volume.volume_id,
                    need_download_url=need_download_url,
                    path=base_path,
                    recursive=recursive,
                )
            except VesslApiException:
                files = []
            for file in files:
                file.path = os.path.relpath(f"/{file.path}", base_path)
            return files

    logger.info("No output volume mounted")


def download_experiment_output_files(
    experiment_number: int,
    dest_path: str = os.path.join(os.getcwd(), "output"),
    worker_number: int = 0,
    **kwargs,
) -> None:
    """Download experiment output files in the default organization/project.
    If you want to override the default organization/project, then pass
    `organization_name` or `project_name` as `**kwargs`.

    Args:
        experiment_number(int): Experiment number.
        dest_path(str): Local download path. Defaults to "./output".
        worker_number(int): Override default worker number (for distributed
            experiments only). Defaults to 0.

    Example:
        ```python
        vessl.download_experiment_output_files(
            experiment_number=23,
        )
        ```
    """
    experiment = read_experiment(experiment_number, **kwargs)

    if experiment.is_local:
        volume_id = experiment.local_execution_spec.output_volume_id
        return copy_volume_file(
            source_volume_id=volume_id,
            source_path="/.",
            dest_volume_id=None,
            dest_path=dest_path,
        )
    if experiment.is_distributed:
        worker_replicas = experiment.distributed_spec.pytorch_spec.worker_replicas
        if worker_number >= worker_replicas:
            raise InvalidParamsError(
                "worker-number: should be less than {}".format(worker_replicas)
            )

        for volume_mount in experiment.volume_mounts_list[worker_number].mounts:
            if volume_mount.source_type == SOURCE_TYPE_OUTPUT:
                return copy_volume_file(
                    source_volume_id=volume_mount.volume.volume_id,
                    source_path="/.",
                    dest_volume_id=None,
                    dest_path=dest_path,
                )
    else:
        for volume_mount in experiment.volume_mounts.mounts:
            if volume_mount.source_type == SOURCE_TYPE_OUTPUT:
                return copy_volume_file(
                    source_volume_id=volume_mount.volume.volume_id,
                    source_path="/.",
                    dest_volume_id=None,
                    dest_path=dest_path,
                )

    logger.info("No output volume mounted")


def terminate_experiment(experiment_number: int, **kwargs) -> ResponseExperimentInfo:
    """Terminate experiment in the default organization/project. If you want to
    override the default organization/project, then pass `organization_name` or
    `project_name` as `**kwargs`.

    Args:
        experiment_number(int): Experiment number.

    Example:
        ```python
        vessl.terminate_experiment(
            experiment_number=23,
        )
        ```
    """
    return vessl_api.experiment_terminate_api(
        experiment_number=experiment_number,
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
    )


def list_github_code_refs(workload_id: int):
    return vessl_api.git_hub_code_refs_api(
        workload_id=workload_id,
    )


def delete_experiment(experiment_number: int, **kwargs):
    """Delete experiment in the default organization/project. If you want to
    override the default organization/project, then pass `organization_name` or
    `project_name` as `**kwargs`.

    Args:
        experiment_number(int): Experiment number.

    Example:
        ```python
        vessl.delete_experiment(
            experiment_number=23,
        )
        ```
    """
    return vessl_api.experiment_delete_api(
        experiment_number=experiment_number,
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
    )



import glob
import os
from tempfile import TemporaryDirectory

from transformers import (
    Trainer,
    TrainerCallback,
    TrainerControl,
    TrainerState,
    TrainingArguments,
)

from vessl.internal import api
from vessl.internal import hyperparameters as hp
from vessl.internal import log
from vessl.model import create_model, upload_model_volume_file
from vessl.util import logger

# keys which are in summary logs
SUMMARY_KEYS = {
    "train_runtime",
    "train_samples_per_second",
    "train_steps_per_second",
    "train_loss",
    "total_flos",
}


class VesslCallback(TrainerCallback):
    """
    A [`TrainerCallback`] that logs metrics, media, model checkpoints to [VESSL AI](https://vessl.ai/).

    Args:
        access_token(str): Access token to override. Defaults to
                `access_token` from `~/.vessl/config`.
        organization_name(str): Organization name to override. Defaults to
                `organization_name` from `~/.vessl/config`.
        project_name(str): Project name to override. Defaults to
            `project name` from `~/.vessl/config`.
        credentials_file(str): Defaults to None.
        force_update_access_token(bool): True if force update access token,
            False otherwise. Defaults to False.
        upload_model(bool): True if upload model after training finishes. Defaults to False.
        repository_name(str): Repository name to upload model. If it is None but upload_model is True,
            raise an exception. Defaults to None.
    """

    def __init__(
        self,
        access_token: str = None,
        organization_name: str = None,
        project_name: str = None,
        credentials_file: str = None,
        force_update_access_token: bool = False,
        upload_model: bool = False,
        repository_name: str = None,
    ) -> None:
        api.configure(
            access_token=access_token,
            organization_name=organization_name,
            project_name=project_name,
            credentials_file=credentials_file,
            force_update_access_token=force_update_access_token,
        )

        self.upload_model = upload_model
        self.repository_name = repository_name

        if self.upload_model and self.repository_name is None:
            raise ValueError("You must specify the repository name when you want to upload model")

    def on_log(
        self,
        args: "TrainingArguments",
        state: "TrainerState",
        control: "TrainerControl",
        logs: dict[str, float] = None,
        **kwargs,
    ) -> None:
        # log metrics in the main process only
        if state.is_world_process_zero:
            # if the current step is the max step, and the summary keys are subset of log keys, just skip logging.
            # It is summary log and it will be handled in on_train_end logs
            log_keys = set(logs.keys())
            if state.max_steps != state.global_step or not SUMMARY_KEYS.issubset(log_keys):
                log(payload=logs, step=state.global_step)

    def on_train_end(
        self, args: "TrainingArguments", state: "TrainerState", control: "TrainerControl", **kwargs
    ):
        """Clean up experiment and upload final model when training ends"""
        if state.is_world_process_zero:
            # get the last log and if it contains train stats such as train_runtime, train_steps_per_seconds, etc,
            # log to hyperparameter section
            summary_log = state.log_history[-1]
            if api.is_in_run_exec_context():
                for k in summary_log.keys():
                    print(f"{k}: {summary_log[k]}")
            else:
                hp.update(summary_log)

            if self.upload_model:
                with TemporaryDirectory() as temp_dir:
                    temp_trainer = Trainer(
                        args=args, model=kwargs.get("model"), tokenizer=kwargs.get("tokenizer")
                    )
                    temp_trainer.save_model(temp_dir)

                    response = create_model(self.repository_name)
                    if response is None:
                        # if model has not been created successfully, the function returns None
                        logger.exception("There was an error while creating model")
                    else:
                        model_number = response.number
                        for file_path in glob.glob(f"{temp_dir}/*"):
                            upload_model_volume_file(
                                self.repository_name,
                                model_number,
                                file_path,
                                os.path.basename(file_path),
                            )



import click

from vessl.cli._base import VesslGroup, vessl_argument
from vessl.cli._util import print_data, print_table
from vessl.util.fmt import format_bool
from vessl.util.prompt import prompt_choices
from vessl.cli.organization import organization_name_option
from vessl.kernel_image import list_kernel_images, read_kernel_image


def image_id_prompter(ctx: click.Context, param: click.Parameter, value: int) -> int:
    images = list_kernel_images()
    return prompt_choices("Cluster", [(x.name, x.id) for x in images])


@click.command(name="image", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command()
@vessl_argument("id", type=click.INT, required=True, prompter=image_id_prompter)
@organization_name_option
def read(id: int):
    image = read_kernel_image(image_id=id)
    print_data(
        {
            "Name": image.name,
            "URL": image.image_url,
            "Type": "Managed" if image.is_savvihub_managed else "Custom",
            "Public": format_bool(image.is_public),
            "Packages": image.packages.split("\n"),
        }
    )


@cli.vessl_command()
@organization_name_option
def list():
    images = list_kernel_images()
    print_table(
        images,
        ["Name", "URL"],
        lambda x: [x.name, x.image_url],
    )



import json
import urllib.request
from sys import version
from typing import List

from vessl.openapi_client.models import OrmKernelImage, ResponseKernelImage
from vessl import vessl_api
from vessl.organization import _get_organization_name


def read_kernel_image(image_id: int) -> ResponseKernelImage:
    """Read the kernel image.

    Args:
        image_id(int): Image ID.

    Example:
        ```python
        vessl.read_kernel_image(
            image_id=1,
        )
        ```
    """
    return vessl_api.kernel_image_read_api(image_id=image_id)


def list_kernel_images(**kwargs) -> List[OrmKernelImage]:
    """List kernel images in the default organization. If you
    want to override the default organization, then pass `organization_name` as
    `**kwargs`.

    Example:
        ```python
        vessl.list_kernel_images()
        ```
    """
    return vessl_api.kernel_image_list_api(
        organization_name=_get_organization_name(**kwargs),
    ).results

def list_vessl_managed_images(repository: str, only_latest_revisions: bool = True):
    try:
        response = urllib.request.urlopen(f"https://quay.io/api/v1/repository/vessl-ai/{repository}?includeTags=true").read().decode()
        if response:
            body = json.loads(response)
            tags_raw = body.get("tags", [])
            tags = [tag for tag in tags_raw]
            if only_latest_revisions:
                tags = filter(lambda tag: "-r" not in tag, tags) # Only latest tags by skipping -rN revision tags
            return tags
        return []
    except Exception as e:
        print(f"Error while fetching vessl managed images for {repository}: {e}")
        return []

def get_recent_framework_and_matchin_cuda_versions(framework: str):
    if framework != "torch" and framework != "tensorflow":
        raise ValueError("Only torch and tensorflow are supported")
    all_tags = list_vessl_managed_images(framework)
    
    framework_cuda_version_map = {}
    for tag in all_tags:
        framework_version = tag.split("-")[0]
        if "cuda" in tag:
            cuda_version = tag.split("cuda")[1]
            if framework_version not in framework_cuda_version_map:
                framework_cuda_version_map[framework_version] = []
            framework_cuda_version_map[framework_version].append(cuda_version)
    return framework_cuda_version_map

def list_framework_supported_cuda_version_for_vessl_managed(framework: str, version: str):
    if framework != "torch" and framework != "tensorflow":
        raise ValueError("Only torch and tensorflow are supported")
    all_tags = list_vessl_managed_images(framework)
    target_versions = []
    for tag in all_tags:
        if version in tag:
            target_versions.append(tag)

    cuda_versions = []
    for tag in target_versions:
        if "cuda" in tag:
            cuda_versions.append(tag.split("cuda")[1])

    return cuda_versions


import os

from watchdog.events import FileSystemEventHandler
from watchdog.observers import Observer

from vessl.util.constant import JWT_PATH


class WatchJWTHandler(FileSystemEventHandler):
    def __init__(self, logger=None, callback=None):
        self.logger = logger
        self.callback = callback

    def on_moved(self, event):
        super(WatchJWTHandler, self).on_moved(event)
        self.logger.debug(f"Moved file: from {event.src_path} to {event.dest_path}")
        if event.dest_path == JWT_PATH:
            with open(JWT_PATH, "r") as f:
                result = f.read().strip()
                self.callback(result)
            self.logger.debug(f"Moved file: {event.dest_path}")

    def on_created(self, event):
        super(WatchJWTHandler, self).on_created(event)
        if event.src_path == JWT_PATH:
            with open(JWT_PATH, "r") as f:
                result = f.read().strip()
                self.callback(result)
            self.logger.debug(f"Created file: {event.src_path}")

    def on_deleted(self, event):
        super(WatchJWTHandler, self).on_deleted(event)
        if event.src_path == JWT_PATH:
            self.logger.warning(f"Deleted file: {event.src_path}")

    def on_modified(self, event):
        super(WatchJWTHandler, self).on_modified(event)
        if event.src_path == JWT_PATH:
            self.logger.warning(f"Deleted file: {event.src_path}")


class WatchJWT:
    def __init__(self, path: str, logger, callback_function):
        self.path = path
        self.observer = Observer()
        self._token = None
        self.callback_function = callback_function
        self.logger = logger

    def run(self):
        event_handler = WatchJWTHandler(logger=self.logger, callback=self.callback_function)
        self.observer.schedule(event_handler, self.path, recursive=True)
        self.observer.start()
        try:
            while self.observer.is_alive():
                self.observer.join(1)

        finally:
            self.observer.stop()
            self.observer.join()



import secrets
import string
from datetime import datetime, timezone
from importlib import import_module

from vessl.util import logger
from vessl.util.exception import ImportPackageError


def parse_time_to_ago(dt: datetime):
    if not dt:
        return "N/A"

    delta = datetime.now(timezone.utc) - dt

    n = delta.total_seconds()
    units = [("second", 60), ("minute", 60), ("hour", 24), ("day", 365), ("year", 9999)]
    for unit_name, unit_size in units:
        if n < unit_size:
            n_int = int(n)
            return f"{n_int} {unit_name}{'s' if n_int != 1 else ''} ago"
        n /= unit_size

    return "ages ago"


def safe_cast(val, to_type, default=None):
    try:
        return to_type(val)
    except (ValueError, TypeError):
        return default


def get_module(name, required=None):
    try:
        return import_module(name)
    except ImportError:
        msg = f"Error importing optional module {name}"
        if required:
            logger.warn(msg)
            raise ImportPackageError(f"{required}")


def generate_uuid(n: int = 8):
    lowercase_alphanumeric = string.ascii_lowercase + string.digits
    return "".join(secrets.choice(lowercase_alphanumeric) for _ in range(n))


def remove_prefix(text, prefix):
    if text.startswith(prefix):
        return text[len(prefix) :]
    return text



import fnmatch
import os
from concurrent.futures import Future
from typing import List

import google.cloud.storage
from requests_futures.sessions import FuturesSession

from vessl.util.file_transmission import (
    FileTransmissionHandler,
    FileTransmissionTracker,
)


class GoogleStorageUploader:
    @classmethod
    def upload(
        cls,
        client: google.cloud.storage.Client,
        source_directory: str,
        bucket_name: str,
        prefix: str = "",
        ignores: List[str] = [],
    ):
        if prefix.startswith("/"):
            prefix = prefix.removeprefix("/")
        handler = FileTransmissionHandler(cls._upload_future, action="upload")
        bucket = google.cloud.storage.Bucket(client, bucket_name)
        for root, dirs, files in os.walk(source_directory, topdown=True):
            # Remove ignored directories
            dirs[:] = [d for d in dirs if not any(fnmatch.fnmatch(d, ignore) for ignore in ignores)]
            for file in files:
                if any(fnmatch.fnmatch(file, ignore) for ignore in ignores):
                    continue
                full_path = os.path.join(root, file)
                relative_path = os.path.relpath(full_path, source_directory)
                bucket_path = os.path.join(prefix, relative_path)
                blob = google.cloud.storage.Blob(bucket_path, bucket, chunk_size=262144)
                signed_url = blob.create_resumable_upload_session(size=os.stat(full_path).st_size)
                handler.add_file(full_path, signed_url, os.stat(full_path).st_size)

        handler.run()

    @classmethod
    def _upload_future(
        cls,
        session: FuturesSession,
        full_path: str,
        url: str,
        tracker: FileTransmissionTracker,
    ) -> Future:
        return session.put(
            url,
            data=open(full_path, "rb"),
            stream=True,
            hooks={"response": cls._upload_hook(full_path, tracker)},
        )

    @staticmethod
    def _upload_hook(full_path, tracker=None):
        def fn(resp, **kwargs):
            if not resp.ok:
                return

            if tracker:
                stat = os.stat(full_path)
                tracker.increase_done_size(stat.st_size)
                tracker.increase_done_count()
                tracker.print_progress()

        return fn



from typing import Optional

import click

from vessl.cli._base import vessl_argument, vessl_option
from vessl.cli._util import print_table, truncate_datetime
from vessl.cli.storage import storage_cli
from vessl.storage.file import VolumeFile
from vessl.storage.volume_v2 import (
    copy_volume_file,
    create_volume,
    delete_volume,
    delete_volume_file,
    list_volume_files,
    list_volumes,
)
from vessl.util.fmt import format_size
from vessl.util.prompt import generic_prompter, prompt_confirm
from vessl.volume import parse_volume_url


@storage_cli.vessl_command(name="list-volumes")
@vessl_option(
    "--storage-name",
    type=click.STRING,
    required=True,
    prompter=generic_prompter("Storage name"),
    help="The name of the storage.",
)
@vessl_option(
    "--keyword",
    type=click.STRING,
    required=False,
    default=None,
    help="Keyword to search for.",
)
@vessl_option(
    "--limit",
    type=click.INT,
    required=False,
    default=None,
)
def list(storage_name: str, keyword: Optional[str], limit: Optional[int]):
    volumes = list_volumes(storage_name=storage_name, keyword=keyword, limit=limit)
    print_table(
        volumes,
        ["Name", "Updated", "Tags"],
        lambda x: [
            x.name,
            truncate_datetime(x.updated_dt),
            [tag.name for tag in x.tags],
        ],
    )


@storage_cli.vessl_command(name="create-volume")
@vessl_argument(
    "name",
    type=click.STRING,
    required=True,
    prompter=generic_prompter("Volume Name"),
)
@vessl_option(
    "--storage-name",
    type=click.STRING,
    required=True,
    prompter=generic_prompter("Storage name"),
    help="The name of the storage.",
)
@vessl_option(
    "--tag",
    type=click.STRING,
    required=False,
    default=(),
    multiple=True,
    help="The tag(s) of the volume.",
)
def create(name: str, storage_name: str, tag: tuple[str, ...]):
    create_volume(name=name, storage_name=storage_name, tags=tag)


@storage_cli.vessl_command(name="delete-volume")
@vessl_argument(
    "name",
    type=click.STRING,
    required=True,
    prompter=generic_prompter("Volume Name"),
)
@vessl_option(
    "--storage-name",
    type=click.STRING,
    required=True,
    prompter=generic_prompter("Storage name"),
    help="The name of the storage.",
)
def delete(name: str, storage_name: str):
    if not prompt_confirm(f"Are you sure to delete volume `{name}`?"):
        return

    delete_volume(name=name, storage_name=storage_name)


@storage_cli.vessl_command()
@vessl_argument(
    "path",
    type=click.STRING,
    required=True,
)
def list_files(path: str):
    """
    List all files in a volume.

    `PATH` must follow the following format: `volume://{STORAGE_NAME}/{VOLUME_NAME}`
    """
    path_ref = parse_volume_url(path)
    files: list[VolumeFile] = list_volume_files(
        storage_name=path_ref.storage_name,
        volume_name=path_ref.volume_name,
        path=path_ref.volume_path,
    )
    print_table(
        files,
        ["Path", "Size"],
        lambda x: [x.path, format_size(x.size)],
    )


@storage_cli.vessl_command()
@vessl_argument(
    "source",
    type=click.STRING,
    required=True,
)
@vessl_argument(
    "dest",
    type=click.STRING,
    required=True,
)
def copy_file(source: str, dest: str):
    """
    Copy a file from source to dest.

    One of `SOURCE` or `DEST` must follow the format: `volume://{STORAGE_NAME}/{VOLUME_NAME}`.
    The other must be a local file path.
    """
    source_ref = parse_volume_url(source, raise_if_not_exists=True)
    dest_ref = parse_volume_url(dest)

    copy_volume_file(source=source_ref, dest=dest_ref)


@storage_cli.vessl_command()
@vessl_argument(
    "path",
    type=click.STRING,
    required=True,
)
@vessl_option("-r", "--recursive", is_flag=True)
def delete_file(path: str, recursive: bool):
    """
    Delete a file in a volume.

    `PATH` must follow the following format: `volume://{STORAGE_NAME}/{VOLUME_NAME}`.
    """
    if not prompt_confirm(f"Are you sure to delete `{path}`?"):
        return

    path_ref = parse_volume_url(path)

    delete_volume_file(
        storage_name=path_ref.storage_name,
        volume_name=path_ref.volume_name,
        path=path_ref.volume_path,
        recursive=recursive,
    )
